@@ -295,6 +295,17 @@ about_dialog_struct *AboutDialogNew(void *core_ptr)
 	gtk_clist_set_column_width(clist, 0, 320);
  */
 	gtk_container_add(GTK_CONTAINER(parent4), w);
+        if(1)
+        {
+            rcstyle = gtk_rc_style_new();
+
+            g_free(rcstyle->font_name);
+            rcstyle->font_name = g_strdup(VMA_GTK_FONT_NAME_TEXT_TERMINAL);
+
+            gtk_widget_modify_style(w, rcstyle);
+
+            GUIRCStyleDeallocUnref(rcstyle);
+        }
 	gtk_widget_show(w);
 
 	/* Add credits to credits clist. */
@@ -306,8 +317,10 @@ about_dialog_struct *AboutDialogNew(void *core_ptr)
 	    heading[0] = g_strdup(credits_lines[i]);
 	    n = gtk_clist_append(clist, heading);
 	    g_free(heading[0]);
-	    if(style != NULL)
+/*	    if(style != NULL)
 		gtk_clist_set_row_style(clist, n, style);
+ */
+            gtk_clist_set_selectable(clist, n, FALSE);
 	    i++;
 	}
 	gtk_clist_thaw(clist);
