@@ -16,12 +16,46 @@
 
 
 /* Message icons. */
-#include "images/icon_error_32x32.xpm"
 #include "images/icon_info_32x32.xpm"
-#include "images/icon_question_32x32.xpm"
 #include "images/icon_warning_32x32.xpm"
+#include "images/icon_error_32x32.xpm"
+#include "images/icon_question_32x32.xpm"
 #include "images/icon_help_32x32.xpm"
 #include "images/icon_wand_32x32.xpm"
+#include "images/icon_search_32x32.xpm"
+#include "images/icon_security_32x32.xpm"
+#include "images/icon_print_32x32.xpm"
+#include "images/icon_sound_32x32.xpm"
+#include "images/icon_bulb_32x32.xpm"
+#include "images/icon_power_32x32.xpm"
+#include "images/icon_linux_32x32.xpm"
+#include "images/icon_terminal_32x32.xpm"
+#include "images/icon_tuning_32x32.xpm"
+#include "images/icon_tools_32x32.xpm"
+#include "images/icon_monitor_32x32.xpm"
+#include "images/icon_clipboard_32x32.xpm"
+#include "images/icon_clipboard_empty_32x32.xpm"
+
+#include "images/icon_file_32x32.xpm"
+#include "images/icon_folder_closed_32x32.xpm"
+#include "images/icon_folder_opened_32x32.xpm"
+#include "images/icon_link2_32x32.xpm"
+#include "images/icon_pipe_32x32.xpm"
+#include "images/icon_device_misc_32x32.xpm"
+#include "images/icon_device_block_32x32.xpm"
+#include "images/icon_device_character_32x32.xpm"
+#include "images/icon_socket_32x32.xpm"
+
+/* Need to work on these.
+#include "images/icon_move_file_32x32.xpm"
+#include "images/icon_copy_file_32x32.xpm"
+ */
+
+#include "images/icon_planet_32x32.xpm"
+#include "images/icon_ftp_32x32.xpm"
+#include "images/icon_chat_32x32.xpm"
+#include "images/icon_file_www_32x32.xpm"
+
 
 /* Button icons. */
 #include "images/icon_ok_20x20.xpm"
@@ -32,33 +66,34 @@
 /* Confirmation dialog structure. */
 typedef struct {
 
-        gbool initialized;
-        gbool map_state;
-        gint last_icon_code;
-
+	gboolean	initialized;
+	gboolean	map_state;
+	gint		last_icon_code;
 	GtkAccelGroup	*accelgrp;
 
-        GtkWidget       *toplevel,
-                        *main_vbox,
-                        *label_hbox,
-                        *icon_pm,  
-                        *icon_fixed,
-                        *label,
-                        *help_vbox,
-                        *help_icon_pm,
-                        *help_icon_fixed,
-                        *help_label,
-                        *button_hbox;
+	GtkWidget	*toplevel,
+			*main_vbox,
+			*label_hbox,
+			*icon_pm,
+			*icon_fixed,
+			*label,
+			*help_vbox,
+			*help_icon_pm,
+			*help_icon_fixed,
+			*help_label,
+			*button_hbox;
 
-        GtkWidget       *ok_btn,
-                        *yes_btn,
-                        *yes_to_all_btn,
-                        *no_btn,
-                        *cancel_btn,
-                        *ignore_btn,
-                        *retry_btn,
-                        *abort_btn,
-                        *help_btn;
+	GtkWidget	*ok_btn,
+			*yes_btn,
+			*yes_to_all_btn,
+			*no_btn,
+			*cancel_btn,
+			*ignore_btn,
+			*retry_btn,
+			*abort_btn,
+			*help_btn;
+
+	GtkWidget	*last_transient_for;	/* Do not reference. */
 
 } cdialog_struct;
 
@@ -70,11 +105,12 @@ static gint CDialogCloseCB(
 );
 static void CDialogButtonCB(GtkWidget *widget, gpointer data);
 
+static u_int8_t **CDialogGetMessageIconDataFromCode(gint icon_code);
 static void CDialogLoadIcon(cdialog_struct *d, u_int8_t **icon_data);
 
 gint CDialogInit(void);
 void CDialogSetTransientFor(GtkWidget *w);
-gbool CDialogIsQuery(void);
+gboolean CDialogIsQuery(void);
 void CDialogBreakQuery(void);
 static void CDialogRemapButtons(
         cdialog_struct *d,
@@ -97,6 +133,15 @@ gint CDialogGetResponseIconData(
         guint show_buttons,             /* Any of CDIALOG_BTNFLAG_*. */
         guint default_button            /* One of CDIALOG_BTNFLAG_*. */
 );
+gint CDialogGetResponseFile(
+        const gchar *title,
+        const gchar *message_path,	/* Message file path. */
+        const gchar *explaination,
+        gint icon_code,
+        guint show_buttons,             /* Any of CDIALOG_BTNFLAG_*. */
+        guint default_button            /* One of CDIALOG_BTNFLAG_*. */
+);
+
 void CDialogMap(void);
 void CDialogUnmap(void);
 void CDialogShutdown(void);
@@ -187,6 +232,125 @@ static void CDialogButtonCB(GtkWidget *widget, gpointer data)
 }
 
 
+/*
+ *      Returns a statically allocated pointer to the icon data
+ *      that corresponds to the given icon_code.
+ */
+static u_int8_t **CDialogGetMessageIconDataFromCode(gint icon_code)
+{
+        u_int8_t **d = (u_int8_t **)icon_info_32x32_xpm;
+
+        switch(icon_code)
+        {
+          case CDIALOG_ICON_INFO:
+            d = (u_int8_t **)icon_info_32x32_xpm;
+            break;
+          case CDIALOG_ICON_WARNING:
+            d = (u_int8_t **)icon_warning_32x32_xpm;
+            break;
+          case CDIALOG_ICON_ERROR:
+            d = (u_int8_t **)icon_error_32x32_xpm;
+            break;
+          case CDIALOG_ICON_QUESTION:
+            d = (u_int8_t **)icon_question_32x32_xpm;
+            break;
+          case CDIALOG_ICON_HELP:
+            d = (u_int8_t **)icon_help_32x32_xpm;
+            break;
+          case CDIALOG_ICON_WIZARD:
+            d = (u_int8_t **)icon_wand_32x32_xpm;
+            break;
+          case CDIALOG_ICON_SEARCH:
+            d = (u_int8_t **)icon_search_32x32_xpm;
+            break;
+          case CDIALOG_ICON_SECURITY:
+            d = (u_int8_t **)icon_security_32x32_xpm;
+            break;
+          case CDIALOG_ICON_PRINTER:
+            d = (u_int8_t **)icon_print_32x32_xpm;
+            break;
+          case CDIALOG_ICON_SPEAKER:
+            d = (u_int8_t **)icon_sound_32x32_xpm;
+            break;
+          case CDIALOG_ICON_BULB:
+            d = (u_int8_t **)icon_bulb_32x32_xpm;
+            break;
+          case CDIALOG_ICON_POWER:
+            d = (u_int8_t **)icon_power_32x32_xpm;
+            break;
+          case CDIALOG_ICON_LINUX:
+            d = (u_int8_t **)icon_linux_32x32_xpm;
+            break;
+          case CDIALOG_ICON_TERMINAL:
+            d = (u_int8_t **)icon_terminal_32x32_xpm;
+            break;
+          case CDIALOG_ICON_SETTINGS:
+            d = (u_int8_t **)icon_tuning_32x32_xpm;
+            break;
+          case CDIALOG_ICON_TOOLS:
+            d = (u_int8_t **)icon_tools_32x32_xpm;
+            break;
+          case CDIALOG_ICON_MONITOR:
+            d = (u_int8_t **)icon_monitor_32x32_xpm;
+            break;
+          case CDIALOG_ICON_CLIPBOARD_EMPTY:
+            d = (u_int8_t **)icon_clipboard_empty_32x32_xpm;
+            break;
+          case CDIALOG_ICON_CLIPBOARD_FULL:
+            d = (u_int8_t **)icon_clipboard_32x32_xpm;
+            break;
+
+          case CDIALOG_ICON_FILE:
+            d = (u_int8_t **)icon_file_32x32_xpm;
+            break;
+          case CDIALOG_ICON_FOLDER_CLOSER:
+            d = (u_int8_t **)icon_folder_closed_32x32_xpm;
+            break;
+          case CDIALOG_ICON_FOLDER_OPENED:
+            d = (u_int8_t **)icon_folder_opened_32x32_xpm;
+            break;
+          case CDIALOG_ICON_LINK:
+            d = (u_int8_t **)icon_link2_32x32_xpm;
+            break;
+          case CDIALOG_ICON_PIPE:
+            d = (u_int8_t **)icon_pipe_32x32_xpm;
+            break;
+          case CDIALOG_ICON_DEVICE:
+            d = (u_int8_t **)icon_device_misc_32x32_xpm;
+            break;
+          case CDIALOG_ICON_DEVICE_BLOCK:
+            d = (u_int8_t **)icon_device_block_32x32_xpm;
+            break;
+          case CDIALOG_ICON_DEVICE_CHARACTER:
+            d = (u_int8_t **)icon_device_character_32x32_xpm;
+            break;
+          case CDIALOG_ICON_SOCKET:
+            d = (u_int8_t **)icon_socket_32x32_xpm;
+            break;
+
+          case CDIALOG_ICON_FILE_MOVE:
+            d = (u_int8_t **)icon_file_32x32_xpm;
+            break;
+          case CDIALOG_ICON_FILE_COPY:
+            d = (u_int8_t **)icon_file_32x32_xpm;
+            break;
+
+          case CDIALOG_ICON_PLANET:
+            d = (u_int8_t **)icon_planet_32x32_xpm;
+            break;
+          case CDIALOG_ICON_FTP:
+            d = (u_int8_t **)icon_ftp_32x32_xpm;
+            break;
+          case CDIALOG_ICON_CHAT:
+            d = (u_int8_t **)icon_chat_32x32_xpm;
+            break;
+          case CDIALOG_ICON_FILE_WWW:
+            d = (u_int8_t **)icon_file_www_32x32_xpm;
+            break;
+        }
+        return(d);
+}
+
 /*
  *	Updates the dialog's icons as needed.
  */
@@ -357,6 +521,7 @@ widget \"*cdialog-tooltips\" style \"cdialog-tooltips-style\"\n\
 	d->initialized = TRUE;
 	d->map_state = FALSE;
 	d->last_icon_code = CDIALOG_ICON_WARNING;
+	d->last_transient_for = NULL;
 
 	/* Parse rc string for help style. */
 	gtk_rc_parse_string(rcstr);
@@ -545,16 +710,16 @@ widget \"*cdialog-tooltips\" style \"cdialog-tooltips-style\"\n\
 	    "Oui",
 #endif
 #ifdef PROG_LANGUAGE_GERMAN
-            "Ja"
+            "Ja",
 #endif
 #ifdef PROG_LANGUAGE_ITALIAN
-            "Si"
+            "Si",
 #endif
 #ifdef PROG_LANGUAGE_NORWEGIAN
-            "Ja"
+            "Ja",
 #endif
 #ifdef PROG_LANGUAGE_PORTUGUESE
-            "Sim"
+            "Sim",
 #endif
 	    NULL
         );
@@ -584,16 +749,16 @@ widget \"*cdialog-tooltips\" style \"cdialog-tooltips-style\"\n\
             "Oui A Tout",
 #endif
 #ifdef PROG_LANGUAGE_GERMAN
-            "Ja Allen"
+            "Ja Allen",
 #endif
 #ifdef PROG_LANGUAGE_ITALIAN
-            "Si Tutto il"
+            "Si Tutto il",
 #endif
 #ifdef PROG_LANGUAGE_NORWEGIAN
-            "Ja Til All"
+            "Ja Til All",
 #endif
 #ifdef PROG_LANGUAGE_PORTUGUESE
-            "Sim A Todo"
+            "Sim A Todo",
 #endif
 	     NULL
         );
@@ -623,16 +788,16 @@ widget \"*cdialog-tooltips\" style \"cdialog-tooltips-style\"\n\
             "Non",
 #endif
 #ifdef PROG_LANGUAGE_GERMAN
-            "Nein"
+            "Nein",
 #endif
 #ifdef PROG_LANGUAGE_ITALIAN
-            "No"
+            "No",
 #endif
 #ifdef PROG_LANGUAGE_NORWEGIAN
-            "Ingen"
+            "Ingen",
 #endif
 #ifdef PROG_LANGUAGE_PORTUGUESE
-            "No"
+            "No",
 #endif
 	    NULL
         );
@@ -662,16 +827,16 @@ widget \"*cdialog-tooltips\" style \"cdialog-tooltips-style\"\n\
             "Annuler",
 #endif
 #ifdef PROG_LANGUAGE_GERMAN
-            "Auf"
+            "Auf",
 #endif
 #ifdef PROG_LANGUAGE_ITALIAN
-            "Annulla"
+            "Annulla",
 #endif
 #ifdef PROG_LANGUAGE_NORWEGIAN
-            "Kanseller"
+            "Kanseller",
 #endif
 #ifdef PROG_LANGUAGE_PORTUGUESE
-            "cancelamento"
+            "Cancelamento",
 #endif
 	    NULL
         );
@@ -705,16 +870,16 @@ widget \"*cdialog-tooltips\" style \"cdialog-tooltips-style\"\n\
             "Nglige",
 #endif
 #ifdef PROG_LANGUAGE_GERMAN
-            "Ignoriert"
+            "Ignoriert",
 #endif
 #ifdef PROG_LANGUAGE_ITALIAN
-            "Ignora"
+            "Ignora",
 #endif
 #ifdef PROG_LANGUAGE_NORWEGIAN
-            "Ignorer"
+            "Ignorer",
 #endif
 #ifdef PROG_LANGUAGE_PORTUGUESE
-            "ignora"
+            "ignora",
 #endif
 	    NULL
         );
@@ -744,16 +909,16 @@ widget \"*cdialog-tooltips\" style \"cdialog-tooltips-style\"\n\
             "Retry",
 #endif
 #ifdef PROG_LANGUAGE_GERMAN
-            "Wiederholt"
+            "Wiederholt",
 #endif
 #ifdef PROG_LANGUAGE_ITALIAN
-            "Retry"
+            "Retry",
 #endif
 #ifdef PROG_LANGUAGE_NORWEGIAN
-            "Omprving"
+            "Omprving",
 #endif
 #ifdef PROG_LANGUAGE_PORTUGUESE
-            "Retry"
+            "Retry",
 #endif
 	    NULL
         );
@@ -783,16 +948,16 @@ widget \"*cdialog-tooltips\" style \"cdialog-tooltips-style\"\n\
             "Avorter",
 #endif
 #ifdef PROG_LANGUAGE_GERMAN
-            "Abort"
+            "Abort",
 #endif
 #ifdef PROG_LANGUAGE_ITALIAN
-            "Abortisce"
+            "Abortisce",
 #endif
 #ifdef PROG_LANGUAGE_NORWEGIAN
-            "Aborter"
+            "Aborter",
 #endif
 #ifdef PROG_LANGUAGE_PORTUGUESE
-            "Aborta"
+            "Aborta",
 #endif
 	    NULL
         );
@@ -820,22 +985,22 @@ widget \"*cdialog-tooltips\" style \"cdialog-tooltips-style\"\n\
             "Help",
 #endif
 #ifdef PROG_LANGUAGE_SPANISH
-            "La Ayuda",
+            "Ayuda",
 #endif
 #ifdef PROG_LANGUAGE_FRENCH
             "l'Aide",
 #endif
 #ifdef PROG_LANGUAGE_GERMAN
-	    "Bricht Hilfe"
+	    "Hilfe",
 #endif
 #ifdef PROG_LANGUAGE_ITALIAN
-            "l'Aiuto"
+            "l'Aiuto",
 #endif
 #ifdef PROG_LANGUAGE_NORWEGIAN
-            "hjelp"
+            "hjelp",
 #endif
 #ifdef PROG_LANGUAGE_PORTUGUESE
-            "Ajuda"
+            "Ajuda",
 #endif
 	    NULL
         );
@@ -886,17 +1051,19 @@ void CDialogSetTransientFor(GtkWidget *w)
 	{
 	    if(w != NULL)
 	    {
-		/* Given widget if not NULL, must be a window. */
 		if(!GTK_IS_WINDOW(GTK_OBJECT(w)))
 		    return;
 
+		if(GTK_WINDOW(w)->modal)
+		    gtk_window_set_modal(GTK_WINDOW(w), FALSE);
+
 		gtk_window_set_modal(
 		    GTK_WINDOW(d->toplevel), TRUE
 		);
-/*		gtk_grab_add(d->toplevel); */
 		gtk_window_set_transient_for(
 		    GTK_WINDOW(d->toplevel), GTK_WINDOW(w)
 		);
+		d->last_transient_for = w;
 	    }
 	    else
 	    {
@@ -906,6 +1073,7 @@ void CDialogSetTransientFor(GtkWidget *w)
 		gtk_window_set_transient_for(
 		    GTK_WINDOW(d->toplevel), NULL
 		);
+		d->last_transient_for = NULL;
 	    }
 	}
 }
@@ -913,7 +1081,7 @@ void CDialogSetTransientFor(GtkWidget *w)
 /*
  *	Returns TRUE if currently blocking for query.
  */
-gbool CDialogIsQuery(void)
+gboolean CDialogIsQuery(void)
 {
 	if(block_loop_level > 0)
 	    return(TRUE);
@@ -1138,36 +1306,17 @@ gint CDialogGetResponse(
 		(explaination == NULL) ? CDIALOG_NO_HELP_STR : explaination
 	    );
 
-	/* Update icon. */
+	/* Need to update message icon? */
 	if(icon_code != d->last_icon_code)
 	{
+            /* Update the last recorded icon code. */
 	    d->last_icon_code = icon_code;
-	    switch(icon_code)
-	    {
-	      case CDIALOG_ICON_ERROR:
-		CDialogLoadIcon(d, (u_int8_t **)icon_error_32x32_xpm);
-		break;
 
-              case CDIALOG_ICON_QUESTION:
-                CDialogLoadIcon(d, (u_int8_t **)icon_question_32x32_xpm);
-                break;
-
-              case CDIALOG_ICON_WARNING:
-                CDialogLoadIcon(d, (u_int8_t **)icon_warning_32x32_xpm);
-                break;
-
-              case CDIALOG_ICON_HELP:
-                CDialogLoadIcon(d, (u_int8_t **)icon_help_32x32_xpm);
-                break;
-
-	      case CDIALOG_ICON_WIZARD:
-		CDialogLoadIcon(d, (u_int8_t **)icon_wand_32x32_xpm);
-                break;
-
-              default:
-                CDialogLoadIcon(d, (u_int8_t **)icon_info_32x32_xpm);
-                break;
-	    }
+            /* Now set the new message icon with the choosen icon data. */
+            CDialogLoadIcon(
+                d,
+                CDialogGetMessageIconDataFromCode(icon_code)
+            );
 	}
 
 	/* Remap buttons. */
@@ -1185,7 +1334,16 @@ gint CDialogGetResponse(
 	if(d->toplevel != NULL)
 	    gtk_widget_queue_resize(d->toplevel);
 
-
+	/* Center toplevel if not transient for. */
+/* GTK_WIN_POS_NONE does not seem to work in "resetting" the position
+	w = d->toplevel;
+	if((w != NULL) ? GTK_IS_WINDOW(w) : FALSE)
+            gtk_window_set_position(
+                GTK_WINDOW(w),
+                (d->last_transient_for != NULL) ?
+                    GTK_WIN_POS_NONE : GTK_WIN_POS_CENTER
+            );
+ */
 	/* Map dialog. */
 	CDialogMap();
 
@@ -1283,6 +1441,16 @@ gint CDialogGetResponseIconData(
         if(d->toplevel != NULL)
             gtk_widget_queue_resize(d->toplevel);
 
+        /* Center toplevel if not transient for. */
+/*
+        w = d->toplevel;
+        if((w != NULL) ? GTK_IS_WINDOW(w) : FALSE)
+            gtk_window_set_position(
+                GTK_WINDOW(w),
+                (d->last_transient_for != NULL) ?
+                    GTK_WIN_POS_NONE : GTK_WIN_POS_CENTER
+            );
+ */
 
         /* Map dialog. */
         CDialogMap();
@@ -1305,6 +1473,66 @@ gint CDialogGetResponseIconData(
         return(response_code);
 }
 
+/*
+ *	Similar to CDialogGetResponse() except that it displays a
+ *	message from the file specified by message_path.
+ */
+gint CDialogGetResponseFile(
+        const gchar *title,
+        const gchar *message_path,	/* Message file path. */
+        const gchar *explaination,
+        gint icon_code,
+        guint show_buttons,             /* Any of CDIALOG_BTNFLAG_*. */
+        guint default_button            /* One of CDIALOG_BTNFLAG_*. */
+)
+{
+	gint status;
+	gchar *buf;
+	FILE *fp;
+
+
+	if(message_path == NULL)
+	    fp = NULL;
+	else
+	    fp = fopen(message_path, "rb");
+	if(fp == NULL)
+	{
+	    buf = g_strdup_printf(
+		"Unable to open file `%s'",
+		message_path
+	    );
+	}
+	else
+	{
+	    gint	buf_len = 2048,
+			bytes_read;
+
+	    buf = g_malloc(buf_len * sizeof(gchar));
+	    if(buf != NULL)
+		bytes_read = fread(buf, sizeof(gchar), buf_len, fp);
+	    else
+		bytes_read = 0;
+	    if(bytes_read < 0)
+		bytes_read = 0;
+	    else if(bytes_read >= buf_len)
+		bytes_read = buf_len - 1;
+	    buf[bytes_read] = '\0';
+
+	    fclose(fp);
+	    fp = NULL;
+	}
+
+	status = CDialogGetResponse(
+	    title, buf, explaination, icon_code,
+	    show_buttons, default_button
+	);
+
+	g_free(buf);
+	buf = NULL;
+
+	return(status);
+}
+
 /*
  *	Maps the confirmation dialog.
  */
@@ -1320,14 +1548,9 @@ void CDialogMap(void)
         if(!d->initialized)
             return;
 
-        if(!d->map_state)
-        {
-            w = d->toplevel;
-            if(w != NULL)
-                gtk_widget_show(w);
-            
-            d->map_state = TRUE;
-        }
+	w = d->toplevel;
+	gtk_widget_show_raise(w);
+	d->map_state = TRUE;
 }
 
 /*
@@ -1383,15 +1606,8 @@ void CDialogShutdown(void)
 	{
 	    /* Begin destroying dialog widgets. */
 
-#define DO_DESTROY_WIDGET       \
-{ \
- if((*w) != NULL) \
- { \
-  GtkWidget *tmp_w = *w; \
-  (*w) = NULL; \
-  gtk_widget_destroy(tmp_w); \
- } \
-}
+#define DO_DESTROY_WIDGET	\
+{ if(*w != NULL) { gtk_widget_destroy(*w); *w = NULL; } }
 
             w = &d->icon_pm;
 	    DO_DESTROY_WIDGET
