@@ -238,8 +238,6 @@ static void ClipboardBrowserSetBusy(clipboard_browser_struct *cb)
 
         gdk_window_set_cursor(w->window, cur);
         gdk_flush();
-
-        return;
 }
 
 /*
@@ -263,8 +261,6 @@ static void ClipboardBrowserSetReady(clipboard_browser_struct *cb)
 
         gdk_window_set_cursor(w->window, NULL);
         gdk_flush();  
-
-        return;
 }
 
 /*
@@ -291,8 +287,6 @@ static void ClipboardBrowserSetStatusMesg(
 		(mesg == NULL) ? "" : mesg
 	    );
 	}
-
-	return;
 }
 
 /*
@@ -415,8 +409,6 @@ static void ClipboardBrowserSetStatusIcon(clipboard_browser_struct *cb)
             /* Set new pixmap. */  
             cb->status_owned_icon_pm = pixmap;
         }
-
-	return;       
 }
 
 /*
@@ -438,8 +430,6 @@ static void ClipboardBrowserDDEClear(clipboard_browser_struct *cb)
 	cb->data_type = CLIPBOARD_DATA_TYPE_ASCII;
 
 	cb->own_buffer = FALSE;
-
-	return;
 }
 
 /*
@@ -470,8 +460,6 @@ static void ClipboardBrowserRecordWidget(
 	}
 
 	(*list)[i] = w;
-
-	return;
 }
 
 /*
@@ -499,8 +487,6 @@ static void ClipboardBrowserClearMCB(GtkWidget *widget, gpointer data)
 	    cb,
 	    ((we_owned_buffer) ? "Buffer cleared" : "Display cleared")
 	);
-
-	return;
 }
 
 /*
@@ -2499,19 +2485,12 @@ void ClipboardBrowserMap(void)
         if(!cb->initialized)
             return;
 
-        if(!cb->map_state)
-        {  
-            w = cb->toplevel;
-            if(w != NULL)
-                gtk_widget_show(w);
-
-            cb->map_state = TRUE;
-        }
+	w = cb->toplevel;
+	gtk_widget_show_raise(w);
+	cb->map_state = TRUE;
 
 	/* Update menus. */
 	ClipboardBrowserUpdateMenus(cb);
-
-	return;
 }
 
 /*
@@ -2537,8 +2516,6 @@ void ClipboardBrowserUnmap(void)
 
             cb->map_state = FALSE;
         }
-
-	return;
 }
 
 /*
