@@ -1911,8 +1911,8 @@ move or copy have been left in their previous location.",
 		/* Sync data on target editor. */
 		EditorSyncData(editor);
 
-                /* Is source editor and new referance model valid? */
-                if((src_editor != NULL) && (src_model_ptr != NULL))
+                /* New referance model valid? */
+                if(src_model_ptr != NULL)
 		{
 		    int tar_model_num = row;
 		    v3d_model_struct *new_model_ptr = V3DModelDup(
@@ -1958,17 +1958,8 @@ move or copy have been left in their previous location.",
                      * model on the target editor's models list. Lets
                      * try to notify the user about what went wrong.
                      */
-                    if(src_editor == NULL)
-                        CDialogGetResponse(
-"Non-critical internal error!",
-"The drag source editor pointer is NULL.",
-"The program encountered a non-critical internal error in\n\
-which some data was inconsistant, the operation has been\n\
-aborted. It is unlikely that any data was lost.",
-                            CDIALOG_ICON_ERROR,
-                            CDIALOG_BTNFLAG_OK | CDIALOG_BTNFLAG_HELP,
-                            CDIALOG_BTNFLAG_OK
-                        );
+
+
 		}
 	    }
 /* Add support for recieving of other drop types here. */
@@ -1978,7 +1969,6 @@ aborted. It is unlikely that any data was lost.",
 	/* Mark as no longer processing. */
 	editor->processing = FALSE;
 
-        return;
 }
 
 /*
