@@ -226,12 +226,7 @@ static void EditorModelCreateDNDListDataRequestCB(
 	if(clist != NULL)
 	{
 	    GList *sel = clist->selection;
-
-	    while(sel != NULL)
-	    {
-		new_model_num = (int)sel->data;
-		sel = sel->next;
-	    }
+	    new_model_num = ((sel != NULL) ? (int)sel->data : -1);
 	}
 
 	/* Get widget 1 (new name entry). */
@@ -332,8 +327,6 @@ static void EditorModelCreateDNDListDataRequestCB(
             );
             data_sent = TRUE;
         }
-
-	return;
 }
 
 /*
@@ -345,7 +338,6 @@ static void EditorModelCreateDNDListDataDeleteCB(
 )
 {
         /* Function no longer used. */
-        return;
 }
 
 /*
@@ -469,7 +461,7 @@ void EditorModelCreateCB(GtkWidget *widget, gpointer data)
 	    free(heading[1]);
             gtk_widget_set_usize(w, 350, 250);
             gtk_clist_column_titles_passive(GTK_CLIST(w));
-            gtk_clist_set_selection_mode(GTK_CLIST(w), GTK_SELECTION_SINGLE);
+            gtk_clist_set_selection_mode(GTK_CLIST(w), GTK_SELECTION_BROWSE);
             gtk_clist_set_row_height(GTK_CLIST(w), VMA_LIST_ROW_SPACING);
 
 	    gtk_clist_set_column_width(
