@@ -890,8 +890,6 @@ void EditorPrimitiveRealize(
 	    );
             break;
 	}
-
-	return;
 }
 
 /*
@@ -979,8 +977,8 @@ void EditorPrimitiveRealizeAllType(
         ma_editor_struct *editor, int model_num, int ptype, gbool rerealize
 )
 {
-	int i;
-	void *p;
+	gint i;
+	gpointer p;
 	v3d_model_struct *model_ptr;
 
 
@@ -1001,13 +999,11 @@ void EditorPrimitiveRealizeAllType(
                 continue;
 
 	    /* Primitive types must match. */
-	    if(ptype != (*(int *)p))
+	    if(ptype != (*(gint *)p))
 		continue;  
 
             EditorPrimitiveRealize(editor, p, rerealize);
         }
-
-        return;
 }
 
 /*
