@@ -10,8 +10,7 @@
 
 typedef struct {
 
-	gbool initialized;
-	gbool map_state;
+	gboolean map_state;
 	GtkAccelGroup *accelgrp;
 	GtkWidget *toplevel;
 	GtkWidget *label;
