@@ -27,6 +27,7 @@ typedef struct {
 
 	gbool initialized;
 	gbool map_state;
+	GdkVisibilityState visibility;
 	gint stop_count;
         GtkWidget *transient_for;	/* Transient for widget. */
 
@@ -66,6 +67,9 @@ static void ProgressDialogDestroyCB(GtkObject *object, gpointer data);
 static gint ProgressDialogCloseCB(
         GtkWidget *widget, GdkEvent *event, gpointer data
 );
+static gint ProgressDialogVisibilityCB(
+        GtkWidget *widget, GdkEventVisibility *visibility, gpointer data
+);
 static gint ProgressDialogAnimationDrawCB(
 	GtkWidget *widget, GdkEventExpose *expose, gpointer data
 );
@@ -88,6 +92,7 @@ static void ProgressDialogIconDeleteAll(
 /* Front ends. */
 gint ProgressDialogInit(void);
 void ProgressDialogSetTransientFor(GtkWidget *w);
+void ProgressDialogSetWMIconData(const u_int8_t **icon_data);
 gbool ProgressDialogIsQuery(void);
 void ProgressDialogBreakQuery(gbool allow_gtk_iteration);
 gint ProgressDialogStopCount(void);
@@ -185,7 +190,23 @@ static gint ProgressDialogCloseCB(
 }
 
 /*
- *	Animation redraw callback.
+ *	Toplevel "visibility_notify_event" signal callback.
+ */
+static gint ProgressDialogVisibilityCB(
+        GtkWidget *widget, GdkEventVisibility *visibility, gpointer data
+)
+{
+	progress_dialog_struct *pd = (progress_dialog_struct *)data;
+	if((visibility == NULL) || (pd == NULL))
+	    return(FALSE);
+
+	pd->visibility = visibility->state;
+
+	return(TRUE);
+}
+
+/*
+ *	Animation redraw and "expose_event" signal callback.
  */
 static gint ProgressDialogAnimationDrawCB(
         GtkWidget *widget, GdkEventExpose *expose, gpointer data
@@ -207,7 +228,9 @@ static gint ProgressDialogAnimationDrawCB(
 
 
 	w = pd->animation_da;
-	if(w == NULL)
+	if((w == NULL) || (pd->visibility == GDK_VISIBILITY_FULLY_OBSCURED) ||
+	   !pd->map_state
+	)
 	    return(TRUE);
 
 	window = w->window;
@@ -748,6 +771,7 @@ gint ProgressDialogInit(void)
 
         pd->initialized = TRUE;
         pd->map_state = FALSE;
+	pd->visibility = GDK_VISIBILITY_FULLY_OBSCURED;
         pd->stop_count = 0;;
 	pd->transient_for = NULL;
 
@@ -758,6 +782,10 @@ gint ProgressDialogInit(void)
 
         /* Toplevel. */
         pd->toplevel = w = gtk_window_new(GTK_WINDOW_DIALOG);
+	gtk_widget_add_events(
+	    w,
+	    GDK_VISIBILITY_NOTIFY_MASK
+	);
         gtk_widget_set_usize(w, PROGRESS_DIALOG_WIDTH, PROGRESS_DIALOG_HEIGHT);
         gtk_window_set_title(GTK_WINDOW(w), "Progress");
 	gtk_widget_realize(w);
@@ -774,15 +802,17 @@ gint ProgressDialogInit(void)
             );
 	    colormap = gdk_window_get_colormap(window);
         }
+	gtk_signal_connect(
+            GTK_OBJECT(w), "visibility_notify_event",
+            GTK_SIGNAL_FUNC(ProgressDialogVisibilityCB), pd
+        );
         gtk_signal_connect(
             GTK_OBJECT(w), "delete_event",
-            GTK_SIGNAL_FUNC(ProgressDialogCloseCB),
-            (gpointer)pd
+            GTK_SIGNAL_FUNC(ProgressDialogCloseCB), pd
         );
         gtk_signal_connect(
             GTK_OBJECT(w), "destroy",
-            GTK_SIGNAL_FUNC(ProgressDialogDestroyCB), 
-            (gpointer)pd
+            GTK_SIGNAL_FUNC(ProgressDialogDestroyCB), pd
         );
         gtk_container_border_width(GTK_CONTAINER(w), 0);
 	gtk_accel_group_attach(accelgrp, GTK_OBJECT(w));
@@ -959,14 +989,15 @@ void ProgressDialogSetTransientFor(GtkWidget *w)
         {
             if(w != NULL)
             {
-                /* Given widget if not NULL, must be a window. */
                 if(!GTK_IS_WINDOW(GTK_OBJECT(w)))
                     return;
 
+		if(GTK_WINDOW(w)->modal)
+		    gtk_window_set_modal(GTK_WINDOW(w), FALSE);
+
                 gtk_window_set_modal(
                     GTK_WINDOW(pd->toplevel), TRUE
                 );
-/*              gtk_grab_add(pd->toplevel); */
                 gtk_window_set_transient_for(
                     GTK_WINDOW(pd->toplevel), GTK_WINDOW(w)
                 );
@@ -983,10 +1014,27 @@ void ProgressDialogSetTransientFor(GtkWidget *w)
 		pd->transient_for = NULL;
             }
         }
-
-        return;
 }
 
+/*
+ *	Explicitly sets the progress dialog's toplevel WM icon.
+ */
+void ProgressDialogSetWMIconData(const u_int8_t **icon_data)
+{
+	GtkWidget *w;
+        progress_dialog_struct *pd = &progress_dialog;
+
+
+        if(!pd->initialized)
+            return;
+
+	w = pd->toplevel;
+        if(w == NULL)
+            return;
+
+        /* Set WM icon for toplevel. */
+        GUISetWMIcon(w->window, (u_int8_t **)icon_data);
+}
 
 /*
  *	Returns TRUE if dialog is mapped.
@@ -1110,13 +1158,7 @@ void ProgressDialogMap(
 	if(!pd->map_state)
 	{
 	    w = pd->toplevel;
-            gdk_flush();
-	    if(w != NULL)
-	    {
-		gtk_widget_show(w);
-		gtk_widget_map(w);
-	    }
-            gdk_flush();
+	    gtk_widget_show_raise(w);
 	    pd->map_state = TRUE;
 	}
 }
@@ -1258,13 +1300,7 @@ void ProgressDialogMapAnimation(
         if(!pd->map_state)
         {
             w = pd->toplevel;
-            gdk_flush();
-            if(w != NULL)
-            {
-                gtk_widget_show(w);
-                gtk_widget_map(w);
-            }
-            gdk_flush();
+	    gtk_widget_show_raise(w);
             pd->map_state = TRUE;
         }
 }
