@@ -5,6 +5,8 @@
 #include <limits.h>
 #include <math.h>
 
+#include "../include/os.h"
+
 #ifdef __MSW__
 # include <windows.h>
 #endif
@@ -78,6 +80,13 @@ void V3DGLProcessModel(
 # define PI	3.14159265359
 #endif
 
+#define SQRT(x)         (((x) > 0.0f) ? sqrt(x) : 0.0f)
+
+#define ATOI(s)         (((s) != NULL) ? atoi(s) : 0)
+#define ATOL(s)         (((s) != NULL) ? atol(s) : 0)
+#define ATOF(s)         (((s) != NULL) ? atof(s) : 0.0f)
+#define STRDUP(s)       (((s) != NULL) ? strdup(s) : NULL)
+
 #define MAX(a,b)        (((a) > (b)) ? (a) : (b))
 #define MIN(a,b)        (((a) < (b)) ? (a) : (b))
 #define CLIP(a,l,h)     (MIN(MAX((a),(l)),(h)))
@@ -86,7 +95,8 @@ void V3DGLProcessModel(
 
 
 /*
- *	Allocates a new v3d_glinterprite_struct with all values reset.
+ *	Allocates a new gl interpritation structure with all values
+ *	reset.
  */
 v3d_glinterprite_struct *V3DGLInterpriteNew(void)
 {
@@ -97,7 +107,8 @@ v3d_glinterprite_struct *V3DGLInterpriteNew(void)
 }
 
 /*
- *	Deallocates the given v3d_glinterprite_struct.
+ *	Deallocates the given gl interpritation structure and all
+ *	its allocated resources.
  */
 void V3DGLInterpriteDelete(v3d_glinterprite_struct *glinterp)
 {
@@ -109,21 +120,27 @@ void V3DGLInterpriteDelete(v3d_glinterprite_struct *glinterp)
 	flags = glinterp->flags;
 
 	/* Deallocate any dynamically allocated members. */
+	/* Heightfield base directory. */
 	if(flags & V3D_GLFLAG_HEIGHTFIELD_BASE_DIR)
 	{
 	    free(glinterp->heightfield_base_dir);
 	    glinterp->heightfield_base_dir = NULL;
 	}
+	/* Texture base directory. */
+	if(flags & V3D_GLFLAG_TEXTURE_BASE_DIR)
+	{
+	    free(glinterp->texture_base_dir);
+	    glinterp->texture_base_dir = NULL;
+	}
 
 	/* Deallocate GL interpritation structure itself. */
 	free(glinterp);
-
-	return;
 }
 
 
 /*
- *	Allocates a new v3d_glresource_struct with all values reset.
+ *	Allocates a new gl resources structure with all values
+ *	reset.
  */
 v3d_glresource_struct *V3DGLResourceNew(void)
 {
@@ -132,7 +149,11 @@ v3d_glresource_struct *V3DGLResourceNew(void)
 	);
 	if(glres != NULL)
 	{
-	    /* Allocate GL interpritation structure. */
+	    /* Allocate dynamically allocated members of the gl resource
+	     * structure.
+	     */
+
+	    /* gl interpritation structure. */
 	    glres->glinterprite = V3DGLInterpriteNew();
 	}
 
@@ -140,14 +161,14 @@ v3d_glresource_struct *V3DGLResourceNew(void)
 }
 
 /*
- *	Allocates a new v3d_glresource_struct with the values initialized
- *	with respect to the given V3D model data.
+ *	Allocates a new gl resources structure with the values
+ *	initialized with respect to the given V3D model data.
  *
  *	If the given v3d_glinterprite_struct *glinterp is not NULL then
- *	it's values will be set to the new v3d_glresource_struct, 
- *	overriding any values defined in the model header items.
+ *	its values will be coppied to the new gl resources structure
+ *	and overrides any values defined in the model header items.
  *
- *      The correct GL context must be binded before calling this
+ *      The correct gl context must be binded before calling this
  *      function.
  */
 v3d_glresource_struct *V3DGLResourceNewFromModelData(
@@ -157,6 +178,7 @@ v3d_glresource_struct *V3DGLResourceNewFromModelData(
 )
 {
 	int i, n;
+	unsigned int interp_flags;
 	int tex_dest_fmt = V3D_TEX_FORMAT_RGBA;
 	void *h;
 	mh_texture_load_struct *h_texture_load;
@@ -167,12 +189,21 @@ v3d_glresource_struct *V3DGLResourceNewFromModelData(
 	if(glres == NULL)
 	    return(NULL);
 
+	/* Get interpritation flags that define what values that are set
+	 * in the gl interpritation structure.
+	 */
+	interp_flags = (glinterp != NULL) ? glinterp->flags : 0;
+
 	/* Get texture base directory. */
 	texture_base_dir = (const char *)V3DMHTextureBaseDirectoryGet(
 	    mh_item, total_mh_items
 	);
+	/* Interpritation structure overrides? */
+	if(interp_flags & V3D_GLFLAG_TEXTURE_BASE_DIR)
+	    texture_base_dir = glinterp->texture_base_dir;
 
-	/* Load textures by itterating through model header items. */
+
+	/* Load textures by iterating through model header items. */
 	for(i = 0; i < total_mh_items; i++)
 	{
 	    h = mh_item[i];
@@ -254,7 +285,8 @@ v3d_glresource_struct *V3DGLResourceNewFromModelData(
 }
 
 /*
- *	Sets the given glres's GL interpritation to the given glinterp.
+ *	Sets the given gl resources structure's gl interpritation
+ *	legend to the given glinterp.
  *
  *	Returns non-zero on error.
  */
@@ -272,7 +304,9 @@ int V3DGLResourceSetInterpritation(
 
 	flags = glinterp->flags;
 
-	/* Get target GL interpritation structure from the given glres. */
+	/* Get target gl interpritation structure from the given gl
+	 * resource structure.
+	 */
 	t_glinterp = glres->glinterprite;
 	if(t_glinterp == NULL)
 	    return(-1);
@@ -321,10 +355,18 @@ int V3DGLResourceSetInterpritation(
 
 	if(flags & V3D_GLFLAG_HEIGHTFIELD_BASE_DIR)
 	{
+	    const char *src_str = glinterp->heightfield_base_dir;
+
 	    free(t_glinterp->heightfield_base_dir);
-	    t_glinterp->heightfield_base_dir = ((glinterp->heightfield_base_dir == NULL) ?
-		NULL : strdup(glinterp->heightfield_base_dir)
-	    );
+	    t_glinterp->heightfield_base_dir = STRDUP(src_str);
+	}
+
+	if(flags & V3D_GLFLAG_TEXTURE_BASE_DIR)
+	{
+            const char *src_str = glinterp->texture_base_dir;
+
+            free(t_glinterp->texture_base_dir);
+            t_glinterp->texture_base_dir = STRDUP(src_str);
 	}
 
 
@@ -335,7 +377,8 @@ int V3DGLResourceSetInterpritation(
 }
 
 /*
- *	Returns the GL interpritation structure from the given glres.
+ *	Returns the gl interpritation structure from the given gl
+ *	resources structure.
  *
  *	The returned value should not be modified or deallocated.
  *
@@ -345,16 +388,14 @@ v3d_glinterprite_struct *V3DGLResourceGetInterpritation(
         v3d_glresource_struct *glres  
 )
 {
-	if(glres == NULL)
-	    return(NULL);
-
-	return(glres->glinterprite);
+	return((glres != NULL) ? glres->glinterprite : NULL);
 }
 
 /*
- *	Deallocates all resources on the given v3d_glresource_struct.
+ *	Deallocates all resources on the given gl resources
+ *	structure.
  *
- *      The correct GL context must be binded before calling this
+ *      The correct gl context must be binded before calling this
  *      function.
  */
 void V3DGLResourceDelete(v3d_glresource_struct *glres)
@@ -364,30 +405,19 @@ void V3DGLResourceDelete(v3d_glresource_struct *glres)
 	if(glres == NULL)
 	    return;
 
-	/* Deallocate and unload all textures. */
+	/* Unload and deallocate all textures. */
 	for(i = 0; i < glres->total_textures; i++)
-	{
 	    V3DTextureDestroy(glres->texture[i]);
-	    glres->texture[i] = NULL;
-	}
-	if(glres->texture != NULL)
-	{
-	    free(glres->texture);
-	    glres->texture = NULL;
-	}
+	free(glres->texture);
+	glres->texture = NULL;
 	glres->total_textures = 0;
 
-	/* Deallocate GL interpritation structure. */
-	if(glres->glinterprite != NULL)
-	{
-	    free(glres->glinterprite);
-	    glres->glinterprite = NULL;
-	}
+	/* Deallocate gl interpritation structure. */
+	V3DGLInterpriteDelete(glres->glinterprite);
+	glres->glinterprite = NULL;
 
 	/* Deallocate structure itself. */
 	free(glres);
-
-	return;
 }
 
 
@@ -442,7 +472,7 @@ static void V3DGLSetNormal(
            ((tn.x != 0.0) || (tn.y != 0.0) || (tn.z != 0.0))
 	)
 	{
-	    double mag = sqrt(
+	    double mag = SQRT(
 		(tn.x * tn.x) + (tn.y * tn.y) + (tn.z * tn.z)
 	    );
 	    if(mag > 0.0)
@@ -464,8 +494,6 @@ static void V3DGLSetNormal(
 	    glNormal3d(tn.x, tn.z, -tn.y);
 	    break;
 	}
-
-	return;
 }
 
 /*
@@ -573,7 +601,7 @@ static void V3DGLSetTexCoord(
             switch(coordinate_axis)
             {
               case V3D_GLCOORDINATE_AXIS_GL:
-/* Need to work on this. */
+                glTexCoord2d(s, t);
                 break;
 
               case V3D_GLCOORDINATE_AXIS_SCIENTIFIC:
@@ -581,8 +609,6 @@ static void V3DGLSetTexCoord(
                 break;
             }   
 	}
-
-	return;
 }
 
 /*
@@ -613,8 +639,6 @@ static void V3DGLSetVertex(
 	    glVertex3d(v->x, v->z, -v->y);
 	    break;
 	}
-
-	return;
 }
 
 
@@ -644,8 +668,6 @@ static void V3DGLProcessModelOtherData(
 	    if(extra_cb != NULL)
 		extra_cb(m, line_ptr, client_data);
 	}
-
-	return;
 }
 
 /*
@@ -679,6 +701,7 @@ static void V3DGLProcessModelStandard(
 	int blending_enabled = FALSE;
 	int set_blend_func = FALSE;
         const char *heightfield_base_dir = NULL;
+	const char *texture_base_dir = NULL;
 
 	int last_begin_ptype = -1;
 
@@ -728,10 +751,12 @@ static void V3DGLProcessModelStandard(
         if(glinterp->flags & V3D_GLFLAG_SET_BLEND_FUNC)
             set_blend_func = glinterp->set_blend_func;
 	if(glinterp->flags & V3D_GLFLAG_HEIGHTFIELD_BASE_DIR)
-	    heightfield_base_dir = (const char *)glinterp->heightfield_base_dir;
+	    heightfield_base_dir = glinterp->heightfield_base_dir;
+	if(glinterp->flags & V3D_GLFLAG_TEXTURE_BASE_DIR)
+	    texture_base_dir = glinterp->texture_base_dir;
 
 
-	/* Itterate through each primitive on the given editor. */
+	/* Iterate through each primitive on the given editor. */
         for(pn = 0; pn < m->total_primitives; pn++)
 	{
             p = m->primitive[pn];
@@ -1208,8 +1233,8 @@ static void V3DGLProcessModelStandard(
 		     */
 		    last_p_texture_orient = NULL;
 
-		    /* Itterate through each loaded texture on the 
-		     * GL resource structure.
+		    /* Iterate through each loaded texture on the 
+		     * gl resources structure.
 		     */
 		    for(tn = 0; tn < glres->total_textures; tn++)
 		    {
@@ -1394,7 +1419,7 @@ static void V3DGLProcessModelStandard(
 	    last_begin_ptype = -1;
 	}
 
-	/* Pop and extranous matrixes. */
+	/* Pop any extranous matrixes. */
 	while(matrix_level > 0)
 	{
 	    glPopMatrix();
@@ -1407,8 +1432,6 @@ static void V3DGLProcessModelStandard(
 
 	if(blending_enabled)
 	    glDisable(GL_BLEND);
-
-	return;
 }
 
 /*
@@ -1453,8 +1476,6 @@ void V3DGLProcessModelExtra(
             );
 	    break;
 	}
-
-	return;
 }
 
 /*
@@ -1467,5 +1488,4 @@ void V3DGLProcessModel(
 )
 {
 	V3DGLProcessModelExtra(glres, m, NULL, NULL);
-	return;
 }
