@@ -37,6 +37,9 @@ v3d_texture_ref_struct *V3DTextureLoadFromFile2D(
 	const char *path, const char *name, int dest_fmt,
         void *client_data, int (*progress_cb)(void *, int, int)
 );
+v3d_texture_ref_struct *V3DTextureLoadFromFile2DPreempt(
+        const char *path, const char *name, int dest_fmt
+);
 v3d_texture_ref_struct *V3DTextureLoadFromData1D(
 	const void *data, const char *name, int width,
         int bits_per_pixel, int dest_fmt,
@@ -52,40 +55,45 @@ void V3DTexturePriority(v3d_texture_ref_struct *t, double priority);
 void V3DTextureDestroy(v3d_texture_ref_struct *t);
 
 
+#define ATOI(s)         (((s) != NULL) ? atoi(s) : 0)
+#define ATOL(s)         (((s) != NULL) ? atol(s) : 0)
+#define ATOF(s)         (((s) != NULL) ? atof(s) : 0.0f)
+#define STRDUP(s)       (((s) != NULL) ? strdup(s) : NULL)
+
 #define MAX(a,b)        (((a) > (b)) ? (a) : (b))
 #define MIN(a,b)        (((a) < (b)) ? (a) : (b))
 #define CLIP(a,l,h)     (MIN(MAX((a),(l)),(h)))
 
-#define TEXTUREIO_TEX_OPTIONS_1D	\
-{ \
+#define TEXTUREIO_TEX_OPTIONS_1D			\
+{							\
  glTexParameteri(GL_TEXTURE_1D, GL_TEXTURE_WRAP_S, GL_REPEAT); \
  glTexParameteri(GL_TEXTURE_1D, GL_TEXTURE_WRAP_T, GL_REPEAT); \
-  glTexParameteri( \
-  GL_TEXTURE_1D, \
-  GL_TEXTURE_MAG_FILTER, \
-  GL_NEAREST \
- ); \
- glTexParameteri( \
-  GL_TEXTURE_1D,\
-  GL_TEXTURE_MIN_FILTER, \
-  GL_NEAREST \
- ); \
+  glTexParameteri(					\
+  GL_TEXTURE_1D,					\
+  GL_TEXTURE_MAG_FILTER,				\
+  GL_NEAREST						\
+ );							\
+ glTexParameteri(					\
+  GL_TEXTURE_1D,					\
+  GL_TEXTURE_MIN_FILTER,				\
+  GL_NEAREST						\
+ );							\
 }
 
-#define TEXTUREIO_TEX_OPTIONS_2D	\
-{ \
+#define TEXTUREIO_TEX_OPTIONS_2D			\
+{							\
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT); \
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT); \
-  glTexParameteri( \
-  GL_TEXTURE_2D, \
-  GL_TEXTURE_MAG_FILTER, \
-  GL_NEAREST \
- ); \
- glTexParameteri( \
-  GL_TEXTURE_2D,\
-  GL_TEXTURE_MIN_FILTER, \
-  GL_NEAREST \
- ); \
+  glTexParameteri(					\
+  GL_TEXTURE_2D,					\
+  GL_TEXTURE_MAG_FILTER,				\
+  GL_NEAREST						\
+ );							\
+ glTexParameteri(					\
+  GL_TEXTURE_2D,					\
+  GL_TEXTURE_MIN_FILTER,				\
+  GL_NEAREST						\
+ );							\
 }
 
 
@@ -616,8 +624,8 @@ v3d_texture_ref_struct *V3DTextureLoadFromFile2D(
 	}
 
 	/* Record other values. */
-	t->name = StringCopyAlloc(name);
-	t->filename = StringCopyAlloc(path);
+	t->name = STRDUP(name);
+	t->filename = STRDUP(path);
 	t->width = cur_width;
 	t->height = cur_width;	/* Height of tile is width. */
 	t->dimensions = 2;
@@ -636,6 +644,289 @@ v3d_texture_ref_struct *V3DTextureLoadFromFile2D(
 	return(t);
 }
 
+/*
+ *	Similar to V3DTextureLoadFromFile2D(), except faster on UNIXes.
+ *
+ *	This uses the TgaReadFromFileFastRGBA() instead of
+ *	TgaReadFromFile().
+ */
+v3d_texture_ref_struct *V3DTextureLoadFromFile2DPreempt(
+        const char *path, const char *name, int dest_fmt
+)
+{
+        int i;
+#ifndef __MSW__
+        struct stat stat_buf;
+#endif
+        v3d_texture_ref_struct *t;
+        GLuint gl_texture_id;
+        u_int8_t *cur_data_ptr, *loaded_data_ptr;
+        int cur_width, cur_height, loaded_width, loaded_height;
+
+	u_int8_t *data;
+	int data_width, data_height;
+
+        u_int8_t *ptr8, *ptrend8;
+        u_int32_t *ptr32, *ptrend32;
+        u_int8_t r, g, b;
+
+
+        if(path == NULL)
+            return(NULL);
+
+#ifndef __MSW__
+        if(stat(path, &stat_buf))
+        {
+            fprintf(stderr,
+                "%s: No such file.\n",
+                path
+            );
+            return(NULL);
+        }
+        if(!S_ISREG(stat_buf.st_mode))
+        {
+            fprintf(stderr,
+                "%s: Not a file.\n",
+                path
+            );
+            return(NULL);
+        }
+#endif
+        /* Load data from file and load to RGBA format. */
+	data = TgaReadFromFileFastRGBA(
+	    path, &data_width, &data_height, 0x00000000
+	);
+	if(data == NULL)
+            return(NULL);
+
+        /* Check if size is big enough. */
+        if(data_width < 2)
+            fprintf(stderr,
+ "%s: Warning: Image size is too small in width.\n",
+                path
+            );
+        if(data_height < 2)
+            fprintf(stderr,
+ "%s: Warning: Image size is too small in height.\n",
+                path
+            );
+
+        /* Get pointer to beginning of loaded data. */
+        cur_data_ptr = loaded_data_ptr = data;
+
+        cur_width = loaded_width = data_width;
+        cur_height = loaded_height = data_height;
+
+        /* Allocate a texture reference structure. */
+        t = (v3d_texture_ref_struct *)calloc(1, sizeof(v3d_texture_ref_struct));
+        if(t == NULL)
+        {
+	    free(data);
+            return(NULL);
+        }
+
+
+        /* Switch destination data format type, need to convert the
+         * loaded data.
+         */
+        switch(dest_fmt)
+        {
+          case V3D_TEX_FORMAT_RGB:
+            /* Convert source RGBA data to destination RGB data. */
+            ptr32 = (u_int32_t *)cur_data_ptr;
+            ptrend32 = ptr32 + (cur_width * cur_height);
+            ptr8 = (u_int8_t *)cur_data_ptr;
+            ptrend8 = ptr8 + (cur_width * cur_height * 3);
+            while(ptr32 < ptrend32)
+            {
+/*              a = (u_int8_t)(((*ptr32) & 0xff000000) >> 24); */
+                b = (u_int8_t)(((*ptr32) & 0x00ff0000) >> 16);
+                g = (u_int8_t)(((*ptr32) & 0x0000ff00) >> 8);
+                r = (u_int8_t)(((*ptr32) & 0x000000ff) >> 0);
+
+                *ptr8++ = r;
+                *ptr8++ = g;
+                *ptr8++ = b;
+
+                ptr32++;
+            }
+            /* Resize the image as needed (cur_data_ptr may !=
+             * loaded_data_ptr).
+             */
+            V3DTextureRescaleImage(
+                loaded_data_ptr,
+                3, GL_RGB,
+                loaded_width, loaded_height,
+                &cur_data_ptr,
+                &cur_width, &cur_height
+            );
+            break;
+
+          case V3D_TEX_FORMAT_RGBA:
+	    /* Source data is already in RGBA format. */
+
+            /* Resize the image as needed (cur_data_ptr may !=
+             * loaded_data_ptr).
+             */
+            V3DTextureRescaleImage(
+                loaded_data_ptr,
+                4, GL_RGBA,
+                loaded_width, loaded_height,
+                &cur_data_ptr,
+                &cur_width, &cur_height
+            );
+            break;
+
+          case V3D_TEX_FORMAT_LUMINANCE:
+            /* Convert source RGBA data to destination LUMINANCE data. */
+            ptr32 = (u_int32_t *)cur_data_ptr;
+            ptrend32 = ptr32 + (cur_width * cur_height);
+            ptr8 = (u_int8_t *)cur_data_ptr;
+            ptrend8 = ptr8 + (cur_width * cur_height);
+            while(ptr32 < ptrend32)
+            {
+/*              a = (u_int8_t)(((*ptr32) & 0xff000000) >> 24); */
+                b = (u_int8_t)(((*ptr32) & 0x00ff0000) >> 16);
+                g = (u_int8_t)(((*ptr32) & 0x0000ff00) >> 8);
+                r = (u_int8_t)(((*ptr32) & 0x000000ff) >> 0);
+
+                *ptr8++ = (u_int8_t)(((int)r + (int)g + (int)b) / 3);
+                ptr32++;
+            }
+            /* Resize the image as needed (cur_data_ptr may !=
+             * loaded_data_ptr).
+             */
+            V3DTextureRescaleImage(
+                loaded_data_ptr,
+                1, GL_LUMINANCE,
+                loaded_width, loaded_height,
+                &cur_data_ptr,
+                &cur_width, &cur_height
+            );
+            break;
+        }
+        /* Is cur_data_ptr still valid from above processing? */
+        if((cur_data_ptr == NULL) ||
+           (cur_width <= 0) ||
+           (cur_height <= 0)
+        )
+        {
+            free(data);
+            if(cur_data_ptr != loaded_data_ptr)
+                free(cur_data_ptr);
+            return(NULL);
+        }
+
+        /* Create each texture frame. */
+        t->total_frames = (int)cur_height / (int)cur_width;
+        if(t->total_frames < 1)
+            t->total_frames = 1;
+        t->data = (void **)calloc(
+            t->total_frames,
+            sizeof(void *)
+        );
+        if(t->data == NULL)
+        {
+            free(data);
+            if(cur_data_ptr != loaded_data_ptr)
+                free(cur_data_ptr);
+            return(NULL);
+        }
+        for(i = 0; i < t->total_frames; i++)
+        {
+            /* Generate a new texture ID. */
+            glGenTextures(1, &gl_texture_id);
+            if(gl_texture_id == 0)
+            {
+                fprintf(stderr,
+                    "%s: Error generating texture\n",
+                    path
+                );
+                continue;
+            }
+
+            /* Actually generate texture and make texture active
+             * (selected).
+             */
+            glBindTexture(GL_TEXTURE_2D, gl_texture_id);
+
+            /* Set parameters of selected texture. */
+            TEXTUREIO_TEX_OPTIONS_2D
+
+            /* Load data we loaded from file into texture by format. */
+            switch(dest_fmt)
+            {
+              case V3D_TEX_FORMAT_RGB:
+                ptr8 = (u_int8_t *)cur_data_ptr;
+                glTexImage2D(
+                    GL_TEXTURE_2D,      /* GL_TEXTURE_2D or GL_PROXY_TEXTURE_2D. */
+                    0,                  /* Level. */
+                    GL_RGB,             /* Internal format. */
+                    cur_width,          /* Width. */
+                    cur_width,          /* Height (same as width). */
+                    0,                  /* Border. */
+                    GL_RGB,             /* Image data format. */
+                    GL_UNSIGNED_BYTE,   /* Image data type. */
+                    /* Pointer to image data. */
+                    (GLubyte *)(ptr8 + (cur_width * cur_width * i * 3))
+                );
+                break;
+
+              case V3D_TEX_FORMAT_RGBA:
+                ptr32 = (u_int32_t *)cur_data_ptr;
+                glTexImage2D(
+                    GL_TEXTURE_2D,      /* GL_TEXTURE_2D or GL_PROXY_TEXTURE_2D. */
+                    0,                  /* Level. */
+                    GL_RGBA,            /* Internal format. */
+                    cur_width,          /* Width. */
+                    cur_width,          /* Height (same as width). */
+                    0,                  /* Border. */
+                    GL_RGBA,            /* Image data format. */
+                    GL_UNSIGNED_BYTE,   /* Image data type. */
+                    /* Pointer to image data. */
+                    (GLubyte *)(ptr32 + (cur_width * cur_width * i))
+                );
+                break;
+
+              case V3D_TEX_FORMAT_LUMINANCE:
+                ptr8 = (u_int8_t *)cur_data_ptr;
+                glTexImage2D(
+                    GL_TEXTURE_2D,      /* GL_TEXTURE_2D or GL_PROXY_TEXTURE_2D. */
+                    0,                  /* Level. */
+                    GL_LUMINANCE8,      /* Internal format. */
+                    cur_width,          /* Width. */
+                    cur_width,          /* Height (same as width). */
+                    0,                  /* Border. */
+                    GL_LUMINANCE,       /* Image data format. */
+                    GL_UNSIGNED_BYTE,   /* Image data type. */
+                    /* Pointer to image data. */
+                    (GLubyte *)(ptr8 + (cur_width * cur_width * i))
+                );
+                break;
+            }
+
+            /* Record this gl texture id as data. */
+            t->data[i] = (void *)gl_texture_id;
+        }
+
+        /* Record other values. */
+        t->name = STRDUP(name);
+        t->filename = STRDUP(path);
+        t->width = cur_width;
+        t->height = cur_width;  /* Height of tile is width. */
+        t->dimensions = 2;
+
+        /* Deallocate loaded data since it is no longer needed, and
+         * also deallocate cur_data_ptr if it does not match
+         * loaded_data_ptr.
+         */
+        free(data);
+        if(cur_data_ptr != loaded_data_ptr)
+            free(cur_data_ptr);
+
+        return(t);
+}
+
 /*
  *      Creates a 1D texture from memory, returning a dynamically
  *	allocated texture reference structure. The pointer to data
@@ -775,7 +1066,7 @@ v3d_texture_ref_struct *V3DTextureLoadFromData1D(
         }
 
         /* Record other values. */
-        t->name = StringCopyAlloc(name);
+        t->name = STRDUP(name);
         t->filename = NULL;
         t->width = width;
         t->height = width;   /* Height of tile is width. */
@@ -997,7 +1288,7 @@ v3d_texture_ref_struct *V3DTextureLoadFromData2D(
         }
 
         /* Record other values. */
-        t->name = StringCopyAlloc(name);
+        t->name = STRDUP(name);
         t->filename = NULL;
         t->width = cur_width;
         t->height = cur_width;	/* Height of tile is width. */
