@@ -9,7 +9,7 @@
 #include "view.h"
 
 #ifndef gbool
-# define gbool	guchar
+# define gbool	gboolean
 #endif
 
 
