@@ -810,7 +810,8 @@ gint VMAPluginManageTimeoutCB(gpointer data)
 
 
                 /* Set up manage value structure. */
-		v.flags = 0;
+		v.flags = VPI_FLAG_MANAGE_CLIENT_DATA;
+		v.client_data = plugin_ptr->client_data;
 
 		/* Call plug-in's manage function. */
                 plugin_ptr->processing = 1;
