@@ -13,7 +13,7 @@
 #include "v3dmodel.h"
 
 #ifndef gbool
-# define gbool	guchar
+# define gbool	gboolean
 #endif
 
 /*
