@@ -256,7 +256,7 @@ vma_status_bar_struct *VMAStatusBarNew(
         /* Progress bar. */
         adj = (GtkAdjustment *)gtk_adjustment_new(0, 1, 100, 0, 0, 0);
         sb->progress_bar = w = gtk_progress_bar_new_with_adjustment(adj);
-        gtk_widget_set_usize(w, 100, -1);
+        gtk_widget_set_usize(w, 100, 20);
         gtk_progress_bar_set_orientation(
             GTK_PROGRESS_BAR(w), GTK_PROGRESS_LEFT_TO_RIGHT
         );
