@@ -44,6 +44,15 @@
 #include "images/icon_options_32x32.xpm"
 
 
+/* Local utilities. */
+static vma_core_struct *VPIGetCoreFromID(vpi_id *id);
+static ma_editor_struct *VPIGetEditorFromID(vpi_id *id, int editor_id);
+
+
+/* Client data set/get. */
+void VPISetClientData(vpi_id *id, void *client_data);
+void *VPIGetClientData(vpi_id *id);
+
 /* Version setting. */
 int VPISetPluginVersion(
         vpi_id *id, int version_major, int version_minor
@@ -104,6 +113,8 @@ char *VPIFileSelectSimple(
         const char *path
 );
 
+void *VPIGetEditorToplevel(vpi_id *id, int editor_id);
+
 
 /* Value fetching abstraction layer. */
 int VPIGetP(vpi_id *id, int code, void **p);
@@ -218,13 +229,20 @@ void VPIEditorModelDelete(
         int model_num
 );
 void VPIEditorModelSelect(
-        vpi_id *id, int editor_id,
-        int model_num
+        vpi_id *id, int editor_id, int model_num
 );
 v3d_model_struct *VPIEditorModelGetSelected(
 	vpi_id *id, int editor_id, int *model_num
 );
-
+void VPIEditorPrimitiveSelect(
+        vpi_id *id, int editor_id, int pn
+);
+int *VPIEditorPrimitiveGetSelected(
+        vpi_id *id, int editor_id, int *total
+);
+void VPIEditorRedraw(vpi_id *id, int editor_id, int refresh_lists);
+gint VPIEditorGetHasChanges(vpi_id *id, int editor_id);
+void VPIEditorSetHasChanges(vpi_id *id, int editor_id, int has_changes);
 
 
 /* Misc functions. */
@@ -263,6 +281,50 @@ int VPIIsRunning(vpi_id *id, int pid);
 int VPIKill(vpi_id *id, int pid, int s);
 
 
+/*
+ *	Returns the pointer to the core structure.
+ */
+static vma_core_struct *VPIGetCoreFromID(vpi_id *id)
+{
+        vma_plugin_struct *pi = VMA_PLUGIN(id);
+	return((pi != NULL) ? (vma_core_struct *)pi->core_ptr : NULL);
+}
+
+/*
+ *	Returns the pointer to the editor referenced by editor_id.
+ */
+static ma_editor_struct *VPIGetEditorFromID(vpi_id *id, int editor_id)
+{
+	int i = editor_id;
+	vma_core_struct *core_ptr = VPIGetCoreFromID(id);
+	if(core_ptr == NULL)
+	    return(NULL);
+
+	if((i < 0) || (i >= core_ptr->total_editors))
+	    return(NULL);
+
+	return(core_ptr->editor[i]);
+}
+
+/*
+ *	Sets the client data that will be given on the manage and
+ *	shutdown functions.
+ */
+void VPISetClientData(vpi_id *id, void *client_data)
+{
+        vma_plugin_struct *pi = VMA_PLUGIN(id);
+	if(pi != NULL)
+	    pi->client_data = client_data;
+}
+
+/*
+ *	Returns the client data set by VPISetClientData().
+ */
+void *VPIGetClientData(vpi_id *id)
+{
+	vma_plugin_struct *pi = VMA_PLUGIN(id);
+	return((pi != NULL) ? pi->client_data : NULL);
+}
 
 /*
  *	Sets the plug-in version, if the given version is newer than
@@ -278,7 +340,7 @@ int VPISetPluginVersion(
         vpi_id *id, int version_major, int version_minor
 )
 {
-	vma_plugin_struct *pi = (vma_plugin_struct *)id;
+	vma_plugin_struct *pi = VMA_PLUGIN(id);
 	if(pi == NULL)
 	    return(-1);
 
@@ -379,7 +441,7 @@ void VPIMessage(
 )
 {
         int icon_code;   
-        vma_plugin_struct *pi = (vma_plugin_struct *)id;
+        vma_plugin_struct *pi = VMA_PLUGIN(id);
 
 
         switch(type)
@@ -408,24 +470,15 @@ void VPIMessage(
                 details,
                 icon_code,  
                 CDIALOG_BTNFLAG_OK |
-                ((details == NULL) ? 0 : CDIALOG_BTNFLAG_HELP),
+                ((details != NULL) ? CDIALOG_BTNFLAG_HELP : 0),
                 CDIALOG_BTNFLAG_OK
             );
         }
         else
         {
-            ma_editor_struct *editor = NULL;
-            vma_core_struct *core_ptr = (vma_core_struct *)pi->core_ptr;
-
-
-	    if(core_ptr != NULL)
-	    {
-                if((editor_id >= 0) && (editor_id < core_ptr->total_editors))
-		    editor = core_ptr->editor[editor_id];
-            }
-
-            if((editor == NULL) ?
-                0 : (editor->initialized && !editor->processing)
+            ma_editor_struct *editor = VPIGetEditorFromID(id, editor_id);
+            if((editor != NULL) ?
+                (editor->initialized && !editor->processing) : FALSE
             )
                 CDialogSetTransientFor(editor->toplevel);
             else
@@ -465,7 +518,7 @@ int VPIQuestion(
 )
 {
 	int status, icon_code;
-        vma_plugin_struct *pi = (vma_plugin_struct *)id;
+        vma_plugin_struct *pi = VMA_PLUGIN(id);
 
 
 	switch(type)
@@ -504,18 +557,9 @@ int VPIQuestion(
 	}
 	else
 	{
-	    ma_editor_struct *editor = NULL;
-	    vma_core_struct *core_ptr = (vma_core_struct *)pi->core_ptr;
-
-
-            if(core_ptr != NULL)
-            {
-                if((editor_id >= 0) && (editor_id < core_ptr->total_editors))
-                    editor = core_ptr->editor[editor_id];
-            }
-
-	    if((editor == NULL) ?
-		0 : (editor->initialized && !editor->processing)
+	    ma_editor_struct *editor = VPIGetEditorFromID(id, editor_id);
+	    if((editor != NULL) ?
+		(editor->initialized && !editor->processing) : FALSE
 	    )
 		CDialogSetTransientFor(editor->toplevel);
 	    else
@@ -584,7 +628,7 @@ int VPIProgress(
 )
 {
 	const u_int8_t **icon_data;
-        vma_plugin_struct *pi = (vma_plugin_struct *)id;
+        vma_plugin_struct *pi = VMA_PLUGIN(id);
 
 
 	/* Check if progress dialog is not mapped. */
@@ -592,17 +636,9 @@ int VPIProgress(
 	{
 	    if(pi != NULL)
 	    {
-		ma_editor_struct *editor = NULL;
-		vma_core_struct *core_ptr = (vma_core_struct *)pi->core_ptr;
-
-		if(core_ptr != NULL)
-		{
-		    if((editor_id >= 0) && (editor_id < core_ptr->total_editors))
-			editor = core_ptr->editor[editor_id];
-		}
-
-		if((editor == NULL) ?
-		    0 : (editor->initialized && !editor->processing)
+		ma_editor_struct *editor = VPIGetEditorFromID(id, editor_id);
+		if((editor != NULL) ?
+		    (editor->initialized && !editor->processing) : FALSE
 		)
 		    ProgressDialogSetTransientFor(editor->toplevel);
 		else
@@ -701,9 +737,8 @@ int VPIFileSelect(
 	vpi_ftype_struct *ftype_ptr;
 	fb_type_struct **fb_ftype = NULL, *fb_ftype_rtn = NULL;
 	gint fb_total_ftypes = 0;
-	vma_core_struct *core_ptr;
-	ma_editor_struct *editor = NULL;
-        vma_plugin_struct *pi = (vma_plugin_struct *)id;
+	ma_editor_struct *editor;
+        vma_plugin_struct *pi = VMA_PLUGIN(id);
 
 
 	/* Reset returns. */
@@ -717,13 +752,8 @@ int VPIFileSelect(
 	if(pi == NULL)
 	    return(status);
 
-	/* Get editor from core structure. */
-	core_ptr = (vma_core_struct *)pi->core_ptr;
-	if(core_ptr != NULL)
-	{
-	    if((editor_id >= 0) && (editor_id < core_ptr->total_editors))
-		editor = core_ptr->editor[editor_id];
-	}
+	/* Get editor. */
+	editor = VPIGetEditorFromID(id, editor_id);
 
 	/* Check if file browser is busy. */
 	if(FileBrowserIsQuery())
@@ -733,7 +763,7 @@ int VPIFileSelect(
 	 * is valid.
 	 */
 	if((editor != NULL) ?
-	    (editor->initialized && !editor->processing) : 0
+	    (editor->initialized && !editor->processing) : FALSE
 	)
 	    FileBrowserSetTransientFor(editor->toplevel);
 	else
@@ -884,6 +914,16 @@ char *VPIFileSelectSimple(
 }
 
 
+/*
+ *      Returns the pointer to the editor's toplevel GtkWidget, the
+ *      returned pointed will be a GtkWindow *.
+ */
+void *VPIGetEditorToplevel(vpi_id *id, int editor_id)
+{
+        ma_editor_struct *editor = VPIGetEditorFromID(id, editor_id);
+	return((editor != NULL) ? editor->toplevel : NULL);
+}
+
 
 /*
  *	Value fetching abstraction layer.
@@ -895,7 +935,7 @@ int VPIGetP(vpi_id *id, int code, void **p)
 	int status;
 	char *s;
         vma_core_struct *core_ptr;
-        vma_plugin_struct *pi = (vma_plugin_struct *)id;
+        vma_plugin_struct *pi = VMA_PLUGIN(id);
 
 
         if(pi == NULL)
@@ -903,7 +943,7 @@ int VPIGetP(vpi_id *id, int code, void **p)
         if(p == NULL)
             return(VPI_ERROR_BAD_VALUE);
 
-        core_ptr = (vma_core_struct *)pi->core_ptr;
+        core_ptr = VPIGetCoreFromID(id);
 
 	switch(code)
 	{
@@ -955,7 +995,7 @@ int VPIGetP(vpi_id *id, int code, void **p)
 int VPIGetS(vpi_id *id, int code, char **s)
 {
         vma_core_struct *core_ptr;
-        vma_plugin_struct *pi = (vma_plugin_struct *)id;
+        vma_plugin_struct *pi = VMA_PLUGIN(id);
 
 
         if(pi == NULL)
@@ -963,7 +1003,7 @@ int VPIGetS(vpi_id *id, int code, char **s)
         if(s == NULL)
             return(VPI_ERROR_BAD_VALUE);
 
-        core_ptr = (vma_core_struct *)pi->core_ptr;
+        core_ptr = VPIGetCoreFromID(id);
 
 	switch(code)
 	{
@@ -1017,7 +1057,7 @@ int VPIGetS(vpi_id *id, int code, char **s)
 int VPIGetI(vpi_id *id, int code, int *i)
 {
 	vma_core_struct *core_ptr;
-	vma_plugin_struct *pi = (vma_plugin_struct *)id;
+	vma_plugin_struct *pi = VMA_PLUGIN(id);
 
 
         if(pi == NULL)
@@ -1025,7 +1065,7 @@ int VPIGetI(vpi_id *id, int code, int *i)
         if(i == NULL)
             return(VPI_ERROR_BAD_VALUE);
 
-	core_ptr = (vma_core_struct *)pi->core_ptr;
+	core_ptr = VPIGetCoreFromID(id);
 
         switch(code)
         {
@@ -1094,7 +1134,7 @@ int VPIGetUI(vpi_id *id, int code, unsigned int *ui)
 	int i;
 	int status;
         vma_core_struct *core_ptr;
-        vma_plugin_struct *pi = (vma_plugin_struct *)id;
+        vma_plugin_struct *pi = VMA_PLUGIN(id);
 
 
         if(pi == NULL)
@@ -1102,7 +1142,7 @@ int VPIGetUI(vpi_id *id, int code, unsigned int *ui)
         if(ui == NULL)
             return(VPI_ERROR_BAD_VALUE);
 
-        core_ptr = (vma_core_struct *)pi->core_ptr;
+        core_ptr = VPIGetCoreFromID(id);
 
         switch(code)
         {
@@ -1138,7 +1178,7 @@ int VPIGetL(vpi_id *id, vpi_code_t code, long *l)
 	int i;
         int status;
         vma_core_struct *core_ptr;
-        vma_plugin_struct *pi = (vma_plugin_struct *)id;
+        vma_plugin_struct *pi = VMA_PLUGIN(id);
 
 
         if(pi == NULL)
@@ -1146,7 +1186,7 @@ int VPIGetL(vpi_id *id, vpi_code_t code, long *l)
         if(l == NULL)
             return(VPI_ERROR_BAD_VALUE);
 
-        core_ptr = (vma_core_struct *)pi->core_ptr;
+        core_ptr = VPIGetCoreFromID(id);
 
         switch(code)
         {
@@ -1182,7 +1222,7 @@ int VPIGetUL(vpi_id *id, vpi_code_t code, unsigned long *ul)
         int i;
         int status;
         vma_core_struct *core_ptr;
-        vma_plugin_struct *pi = (vma_plugin_struct *)id;
+        vma_plugin_struct *pi = VMA_PLUGIN(id);
 
 
         if(pi == NULL)
@@ -1190,7 +1230,7 @@ int VPIGetUL(vpi_id *id, vpi_code_t code, unsigned long *ul)
         if(ul == NULL)
             return(VPI_ERROR_BAD_VALUE);
 
-        core_ptr = (vma_core_struct *)pi->core_ptr;
+        core_ptr = VPIGetCoreFromID(id);
 
         switch(code)
         {
@@ -1231,7 +1271,7 @@ int VPIGetF(vpi_id *id, int code, float *f)
 	unsigned long ul;
         int status;
         vma_core_struct *core_ptr;
-        vma_plugin_struct *pi = (vma_plugin_struct *)id;
+        vma_plugin_struct *pi = VMA_PLUGIN(id);
 
 
         if(pi == NULL)
@@ -1239,7 +1279,7 @@ int VPIGetF(vpi_id *id, int code, float *f)
         if(f == NULL)
             return(VPI_ERROR_BAD_VALUE);
 
-        core_ptr = (vma_core_struct *)pi->core_ptr;
+        core_ptr = VPIGetCoreFromID(id);
 
         switch(code)
         {
@@ -1287,7 +1327,7 @@ int VPIGetD(vpi_id *id, int code, double *d)
 	unsigned long ul;
         int status;
         vma_core_struct *core_ptr;
-        vma_plugin_struct *pi = (vma_plugin_struct *)id;
+        vma_plugin_struct *pi = VMA_PLUGIN(id);
 
 
         if(pi == NULL)
@@ -1295,7 +1335,7 @@ int VPIGetD(vpi_id *id, int code, double *d)
         if(d == NULL)
             return(VPI_ERROR_BAD_VALUE);
 
-        core_ptr = (vma_core_struct *)pi->core_ptr;
+        core_ptr = VPIGetCoreFromID(id);
 
         switch(code)
         {
@@ -1386,7 +1426,7 @@ int VPISetOperationCB(
         void *client_data
 )
 {
-	vma_plugin_struct *pi = (vma_plugin_struct *)id;
+	vma_plugin_struct *pi = VMA_PLUGIN(id);
 
 
 	if(pi == NULL)
@@ -1454,7 +1494,7 @@ void VPIUnsetOperationCB(
 int VPIImportSetFileType(vpi_id *id, vpi_ftype_struct *ftype)
 {
         int i;
-        vma_plugin_struct *pi = (vma_plugin_struct *)id;                  
+        vma_plugin_struct *pi = VMA_PLUGIN(id);                  
 	if((pi == NULL) || (ftype == NULL))
             return(VPI_ERROR);
 
@@ -1497,7 +1537,7 @@ int VPIImportSetFileType(vpi_id *id, vpi_ftype_struct *ftype)
 void VPIImportClearFileTypes(vpi_id *id)
 {
 	int i;
-	vma_plugin_struct *pi = (vma_plugin_struct *)id;
+	vma_plugin_struct *pi = VMA_PLUGIN(id);
 	if(pi == NULL)
 	    return;
 
@@ -1511,8 +1551,6 @@ void VPIImportClearFileTypes(vpi_id *id)
 	free(pi->import_ftype_name);
 	pi->import_ftype_name = NULL;
 	pi->total_import_ftypes = 0;
-
-	return;
 }
 
 /*
@@ -1525,7 +1563,7 @@ int VPIImportSetFCheckFunc(
         void *client_data
 )
 {
-        vma_plugin_struct *pi = (vma_plugin_struct *)id;
+        vma_plugin_struct *pi = VMA_PLUGIN(id);
         if(pi == NULL)
             return(VPI_ERROR);
 
@@ -1545,7 +1583,7 @@ int VPIImportSetFCheckFunc(
 int VPIExportSetFileType(vpi_id *id, vpi_ftype_struct *ftype)
 {
         int i;
-        vma_plugin_struct *pi = (vma_plugin_struct *)id;
+        vma_plugin_struct *pi = VMA_PLUGIN(id);
 	if((pi == NULL) || (ftype == NULL))
 	    return(VPI_ERROR);
 
@@ -1588,7 +1626,7 @@ int VPIExportSetFileType(vpi_id *id, vpi_ftype_struct *ftype)
 void VPIExportClearFileTypes(vpi_id *id)
 {
         int i;
-        vma_plugin_struct *pi = (vma_plugin_struct *)id;
+        vma_plugin_struct *pi = VMA_PLUGIN(id);
         if(pi == NULL)
             return;
 
@@ -1602,8 +1640,6 @@ void VPIExportClearFileTypes(vpi_id *id)
         free(pi->export_ftype_name);
         pi->export_ftype_name = NULL;
         pi->total_export_ftypes = 0;
-
-        return;
 }
 
 
@@ -1612,14 +1648,12 @@ void VPIExportClearFileTypes(vpi_id *id)
  */
 void VPIRenderSetLabel(vpi_id *id, const char *label)
 {
-        vma_plugin_struct *pi = (vma_plugin_struct *)id;
+        vma_plugin_struct *pi = VMA_PLUGIN(id);
         if(pi == NULL)
             return;
 
 	free(pi->render_label);
 	pi->render_label = ((label == NULL) ? NULL : strdup(label));
-
-	return;
 }
 
 
@@ -1750,7 +1784,7 @@ void VPIAppearanceSet(vpi_id *id, const vpi_appearance_struct *v)
 {
 	GdkWindow *window = (GdkWindow *)GDK_ROOT_PARENT();
         GtkStyle *style = gtk_widget_get_default_style();
-	vma_plugin_struct *pi = (vma_plugin_struct *)id;
+	vma_plugin_struct *pi = VMA_PLUGIN(id);
 
 
 	if((pi == NULL) || (v == NULL))
@@ -1812,8 +1846,6 @@ void VPIAppearanceSet(vpi_id *id, const vpi_appearance_struct *v)
 
 
 	}
-
-	return;
 }
 
 /*
@@ -1823,7 +1855,7 @@ void VPIAppearanceSet(vpi_id *id, const vpi_appearance_struct *v)
  */
 vpi_appearance_struct *VPIAppearanceGet(vpi_id *id)
 {
-	vma_plugin_struct *pi = (vma_plugin_struct *)id;
+	vma_plugin_struct *pi = VMA_PLUGIN(id);
 	vpi_appearance_struct *v;
 
 
@@ -2119,34 +2151,26 @@ int VPIEditorGetHeader(
         void ***mh_item, int *total_mh_items
 )
 {
-	vma_plugin_struct *pi = (vma_plugin_struct *)id;
-	int i = editor_id;
-	vma_core_struct *core_ptr;
+	vma_plugin_struct *pi = VMA_PLUGIN(id);
 	ma_editor_struct *editor;
 
 
         if(mh_item != NULL)
-            (*mh_item) = NULL;
+            *mh_item = NULL;
         if(total_mh_items != NULL)
-            (*total_mh_items) = 0;
+            *total_mh_items = 0;
 
 	if(pi == NULL)
 	    return(VPI_ERROR);
 
-        core_ptr = (vma_core_struct *)pi->core_ptr;
-        if(core_ptr == NULL)
-            return(VPI_ERROR);
-
-        if((i < 0) || (i >= core_ptr->total_editors))
+	editor = VPIGetEditorFromID(id, editor_id);
+        if(editor == NULL)
             return(VPI_ERROR_NOT_FOUND);
-	editor = core_ptr->editor[i];
-	if(editor == NULL)
-	    return(VPI_ERROR);
 
         if(mh_item != NULL)
-            (*mh_item) = editor->mh_item;
+            *mh_item = editor->mh_item;
         if(total_mh_items != NULL)
-            (*total_mh_items) = editor->total_mh_items;
+            *total_mh_items = editor->total_mh_items;
 
 	return(VPI_SUCCESS);
 }
@@ -2161,34 +2185,26 @@ int VPIEditorGetModels(
         v3d_model_struct ***model, int *total_models
 )
 {
-        vma_plugin_struct *pi = (vma_plugin_struct *)id;
-        int i = editor_id;
-	vma_core_struct *core_ptr;
+        vma_plugin_struct *pi = VMA_PLUGIN(id);
 	ma_editor_struct *editor;
 
 
         if(model != NULL)
-            (*model) = NULL;
+            *model = NULL;
         if(total_models != NULL)
-            (*total_models) = 0;
+            *total_models = 0;
 
         if(pi == NULL)
             return(VPI_ERROR);
 
-	core_ptr = (vma_core_struct *)pi->core_ptr;
-	if(core_ptr == NULL)
-	    return(VPI_ERROR);
-
-	if((i < 0) || (i >= core_ptr->total_editors))
-	    return(VPI_ERROR_NOT_FOUND);
-        editor = core_ptr->editor[i];
+        editor = VPIGetEditorFromID(id, editor_id);
         if(editor == NULL)
-            return(VPI_ERROR);
+            return(VPI_ERROR_NOT_FOUND);
 
 	if(model != NULL)
-	    (*model) = editor->model;
+	    *model = editor->model;
 	if(total_models != NULL)
-	    (*total_models) = editor->total_models;
+	    *total_models = editor->total_models;
 
 	return(VPI_SUCCESS);
 }
@@ -2202,9 +2218,7 @@ int VPIEditorGetCamera(
         vpi_id *id, int editor_id, vpi_camera_struct **cam
 )
 {
-        vma_plugin_struct *pi = (vma_plugin_struct *)id;
-        int i = editor_id;
-        vma_core_struct *core_ptr;
+        vma_plugin_struct *pi = VMA_PLUGIN(id);
         ma_editor_struct *editor;
 	vma_view3d_struct *view3d;
 	static vpi_camera_struct s_cam;
@@ -2214,20 +2228,14 @@ int VPIEditorGetCamera(
             return(VPI_ERROR);
 
 	/* Set return pointer. */
-	(*cam) = &s_cam;
+	*cam = &s_cam;
 
 	/* Reset return buffer by just resetting the flags. */
 	s_cam.flags = 0;
 
-        core_ptr = (vma_core_struct *)pi->core_ptr;
-        if(core_ptr == NULL)
-            return(VPI_ERROR);
-
-        if((i < 0) || (i >= core_ptr->total_editors))
-            return(VPI_ERROR_NOT_FOUND);
-        editor = core_ptr->editor[i];
+        editor = VPIGetEditorFromID(id, editor_id);
         if(editor == NULL)
-            return(VPI_ERROR);
+            return(VPI_ERROR_NOT_FOUND);
 
 	/* Get pointer to first 3d view. */
 	view3d = ((VMA_MAX_3D_VIEWS_PER_EDITOR > 0) ?
@@ -2236,7 +2244,7 @@ int VPIEditorGetCamera(
 	if(view3d == NULL)
 	    return(VPI_ERROR);
 
-	if(1)
+	if(TRUE)
 	{
 	    double mag;
 
@@ -2290,9 +2298,8 @@ int VPIEditorGetLight(
         int light_num                   /* Get values from this light. */
 )
 {
-        vma_plugin_struct *pi = (vma_plugin_struct *)id;
-        int i = editor_id;
-        vma_core_struct *core_ptr;
+        vma_plugin_struct *pi = VMA_PLUGIN(id);
+	int i;
         ma_editor_struct *editor;
 	vma_light_struct *vlight_ptr;
 	static vpi_light_struct s_light;
@@ -2302,20 +2309,14 @@ int VPIEditorGetLight(
             return(VPI_ERROR);
 
         /* Set return pointer. */
-        (*light) = &s_light;
+        *light = &s_light;
 
         /* Reset return buffer by resetting the flags. */
 	s_light.flags = 0;
 
-        core_ptr = (vma_core_struct *)pi->core_ptr;
-        if(core_ptr == NULL)
-            return(VPI_ERROR);
-
-        if((i < 0) || (i >= core_ptr->total_editors))
-            return(VPI_ERROR_NOT_FOUND);
-        editor = core_ptr->editor[i];
+        editor = VPIGetEditorFromID(id, editor_id);
         if(editor == NULL)
-            return(VPI_ERROR);
+            return(VPI_ERROR_NOT_FOUND);
 
 	i = light_num;
 	if((i < 0) || (i >= VMA_LIGHTS_MAX))
@@ -2381,7 +2382,7 @@ int VPIEditorGetLights(
         vpi_id *id, int editor_id, vpi_light_struct **light, int *total_lights
 )
 {
-        vma_plugin_struct *pi = (vma_plugin_struct *)id;
+        vma_plugin_struct *pi = VMA_PLUGIN(id);
         int i, nlights;
 	vpi_light_struct *light_rtn;
 	static vpi_light_struct slight[VMA_LIGHTS_MAX];
@@ -2452,8 +2453,8 @@ int VPIEditorGetLights(
 	}
 
 	/* Update returns. */
-	(*light) = slight;
-	(*total_lights) = nlights;
+	*light = slight;
+	*total_lights = nlights;
 
 	return(VPI_SUCCESS);
 }
@@ -2466,21 +2467,13 @@ v3d_model_struct *VPIEditorGetModelByNumber(
         vpi_id *id, int editor_id, int model_num
 )
 {
-        vma_plugin_struct *pi = (vma_plugin_struct *)id;
-        int i = editor_id;
-        vma_core_struct *core_ptr;
+        vma_plugin_struct *pi = VMA_PLUGIN(id);
         ma_editor_struct *editor;
 
         if(pi == NULL)
             return(NULL);
 
-        core_ptr = (vma_core_struct *)pi->core_ptr;
-        if(core_ptr == NULL)
-            return(NULL);
-
-        if((i < 0) || (i >= core_ptr->total_editors))
-            return(NULL);
-        editor = core_ptr->editor[i];
+        editor = VPIGetEditorFromID(id, editor_id);
         if(editor == NULL)
             return(NULL);
 
@@ -2502,28 +2495,21 @@ v3d_model_struct *VPIEditorModelNew(
 	int type, const char *name
 )
 {
-        vma_plugin_struct *pi = (vma_plugin_struct *)id;
-        int i = editor_id, n;
+        vma_plugin_struct *pi = VMA_PLUGIN(id);
+        int i, n;
 	GtkCList *models_clist;
-        vma_core_struct *core_ptr;
 	ma_editor_struct *editor;
 	v3d_model_struct *model_ptr = NULL;
 	gchar *val[1];
 
 
 	if(model_num != NULL)
-	    (*model_num) = -1;
+	    *model_num = -1;
 
         if(pi == NULL)
             return(NULL);
 
-        core_ptr = (vma_core_struct *)pi->core_ptr;
-        if(core_ptr == NULL)
-            return(NULL);
-
-        if((i < 0) || (i >= core_ptr->total_editors))
-            return(NULL);
-        editor = core_ptr->editor[i];
+        editor = VPIGetEditorFromID(id, editor_id);
         if(editor == NULL)
             return(NULL);
 
@@ -2654,27 +2640,19 @@ v3d_model_struct *VPIEditorModelNew(
  *      Deletes the specified model from the specified editor.
  */
 void VPIEditorModelDelete(
-        vpi_id *id, int editor_id,
-	int model_num
+        vpi_id *id, int editor_id, int model_num
 )
 {
-        vma_plugin_struct *pi = (vma_plugin_struct *)id;
-        int i = editor_id, n;
+        vma_plugin_struct *pi = VMA_PLUGIN(id);
+        int i, n;
         GtkCList *models_clist;
-        vma_core_struct *core_ptr;   
         ma_editor_struct *editor;
 
 
         if(pi == NULL)   
             return;
 
-        core_ptr = (vma_core_struct *)pi->core_ptr;
-        if(core_ptr == NULL)
-            return;
-
-        if((i < 0) || (i >= core_ptr->total_editors))
-            return;
-        editor = core_ptr->editor[i];
+        editor = VPIGetEditorFromID(id, editor_id);
         if(editor == NULL)
             return;
 
@@ -2726,63 +2704,51 @@ void VPIEditorModelDelete(
         EditorUpdateMenus(editor);
         EditorUpdateAllViewMenus(editor);
         EditorRedrawAllViews(editor);
-
-	return;
 }
 
 
 /*      
- *      Selects the specified model on the specified editor.
+ *      Selects the specified model on the specified editor, if
+ *	model_num is -1 then all models will be unselected.
  */
 void VPIEditorModelSelect(
-        vpi_id *id, int editor_id,
-        int model_num
+        vpi_id *id, int editor_id, int model_num
 )
 {
-        vma_plugin_struct *pi = (vma_plugin_struct *)id;
-        int i = editor_id;
-        GtkCList *models_clist;
-        vma_core_struct *core_ptr;
+        vma_plugin_struct *pi = VMA_PLUGIN(id);
+        GtkCList *clist;
         ma_editor_struct *editor;
 
 
         if(pi == NULL)
             return;
-            
-        core_ptr = (vma_core_struct *)pi->core_ptr;
-        if(core_ptr == NULL)
-            return;
 
-        if((i < 0) || (i >= core_ptr->total_editors))
-            return;
-        editor = core_ptr->editor[i];
+        editor = VPIGetEditorFromID(id, editor_id);
         if(editor == NULL)
             return;
 
         if(!editor->initialized || editor->processing)
             return;
 
-	models_clist = (GtkCList *)editor->models_list;
-	if(models_clist == NULL)
+	clist = (GtkCList *)editor->models_list;
+	if(clist == NULL)
 	    return;
 
 	/* Unselect all? */
 	if(model_num < 0)
 	{
-	    gtk_clist_unselect_all(models_clist);
+	    gtk_clist_unselect_all(clist);
 	}
 	/* Reselect? */
 	else if(model_num == EditorSelectedModelIndex(editor))
 	{
-	    gtk_clist_unselect_all(models_clist);
-	    gtk_clist_select_row(models_clist, model_num, 0);
+	    gtk_clist_unselect_all(clist);
+	    gtk_clist_select_row(clist, model_num, 0);
 	}
         else
         {
-            gtk_clist_select_row(models_clist, model_num, 0);
+            gtk_clist_select_row(clist, model_num, 0);
         }
-
-        return;
 }
 
 /*
@@ -2793,26 +2759,19 @@ v3d_model_struct *VPIEditorModelGetSelected(
         vpi_id *id, int editor_id, int *model_num
 )
 {
-        vma_plugin_struct *pi = (vma_plugin_struct *)id;
-        int i = editor_id;
-        vma_core_struct *core_ptr;
+        vma_plugin_struct *pi = VMA_PLUGIN(id);
+	gint i;
         ma_editor_struct *editor;
 	v3d_model_struct *model_ptr;
 
 
 	if(model_num != NULL)
-	    (*model_num) = -1;
+	    *model_num = -1;
 
         if(pi == NULL)
             return(NULL);
 
-        core_ptr = (vma_core_struct *)pi->core_ptr;
-        if(core_ptr == NULL)
-            return(NULL);
-
-        if((i < 0) || (i >= core_ptr->total_editors))
-            return(NULL);
-        editor = core_ptr->editor[i];
+        editor = VPIGetEditorFromID(id, editor_id);
         if(editor == NULL)
             return(NULL);
 
@@ -2825,11 +2784,190 @@ v3d_model_struct *VPIEditorModelGetSelected(
 	    return(NULL);
 
 	if(model_num != NULL)
-	    (*model_num) = i;
+	    *model_num = i;
 
 	return(model_ptr);
 }
 
+/*
+ *      Instructs the editor to (re)select the specified primitive if it
+ *      is valid.  If pn is -1 then all primitives will be unselected.
+ */
+void VPIEditorPrimitiveSelect(
+        vpi_id *id, int editor_id, int pn
+)
+{
+        vma_plugin_struct *pi = VMA_PLUGIN(id);
+        GtkCList *clist;
+        ma_editor_struct *editor;
+
+
+        if(pi == NULL)
+            return;
+
+        editor = VPIGetEditorFromID(id, editor_id);
+        if(editor == NULL)
+            return;
+
+        if(!editor->initialized || editor->processing)
+            return;
+
+        clist = (GtkCList *)editor->primitives_list;
+        if(clist == NULL)
+            return;
+
+        /* Unselect all? */
+        if(pn < 0)
+        {
+            gtk_clist_unselect_all(clist);
+        }
+	else
+	{
+	    gint i;
+
+	    /* Check if primitive is already selected. */
+	    for(i = 0; i < editor->total_selected_primitives; i++)
+	    {
+		if(editor->selected_primitive[i] == pn)
+		    break;
+	    }
+	    /* Already selected? */
+	    if(i < editor->total_selected_primitives)
+		gtk_clist_unselect_row(clist, pn, 0);
+
+	    /* Select new row. */
+	    gtk_clist_select_row(clist, pn, 0);
+        }
+}
+
+/*
+ *      Returns the curret selected primitives on the editor or NULL.
+ *      The returned pointer must be free'ed by the calling function.
+ */
+int *VPIEditorPrimitiveGetSelected(
+        vpi_id *id, int editor_id, int *total
+)
+{
+        vma_plugin_struct *pi = VMA_PLUGIN(id);
+        gint t, *s;
+        ma_editor_struct *editor;
+
+
+        if(total != NULL)
+            *total = 0;
+
+        if(pi == NULL)
+            return(NULL);
+
+        editor = VPIGetEditorFromID(id, editor_id);
+        if(editor == NULL)
+            return(NULL);
+
+	/* No primitives selected? */
+	if(editor->selected_primitive == NULL)
+	    return(NULL);
+
+	/* Get total number of primitives selected. */
+	t = editor->total_selected_primitives;
+	if(total != NULL)
+	    *total = t;
+
+	/* Allocate list of selected primitives if there are any selected. */
+	if(t > 0)
+	{
+	    s = (gint *)g_malloc(t * sizeof(gint));
+	    if(s != NULL)
+		memcpy(s, editor->selected_primitive, t * sizeof(gint));
+	}
+	else
+	{
+	    s = NULL;
+	}
+
+	return(s);
+}
+
+/*
+ *	Redraw editor.
+ *
+ *	If refresh_lists is VPI_TRUE then the lists will be refreshed.
+ */
+void VPIEditorRedraw(vpi_id *id, int editor_id, int refresh_lists)
+{
+	gint model_num;
+	v3d_model_struct *m;
+	ma_editor_struct *editor = VPIGetEditorFromID(id, editor_id);
+        if(editor == NULL)
+	    return;
+
+        /* Get pointer to selected model if any. */
+        model_num = EditorSelectedModelIndex(editor);
+        m = V3DModelListGetPtr(
+            editor->model, editor->total_models, model_num
+        );
+
+	/* One primitive selected? */
+        if((m != NULL) && (editor->total_selected_primitives == 1))
+        {
+            gint pn = editor->selected_primitive[0];
+            gpointer p;
+
+	    if((pn >= 0) && (pn < m->total_primitives))
+		p = m->primitive[pn];
+	    else
+		p = NULL;
+
+	    if((p != NULL) && refresh_lists)
+	    {
+		EditorListDeleteValuesG(editor);
+		EditorListAddValuesRG(editor, p);
+            }
+	}
+
+	/* Update menus. */
+        EditorUpdateMenus(editor);
+	EditorUpdateAllViewMenus(editor);
+
+	/* Redraw views if mapped. */
+	if(editor->map_state)
+	{
+	    EditorRedrawAllViews(editor);
+	}
+}
+
+/*
+ *	Returns VPI_TRUE if the editor's has_changes flag is set,
+ *	otherwise returns VPI_FALSE.
+ */
+gint VPIEditorGetHasChanges(vpi_id *id, int editor_id)
+{
+        ma_editor_struct *editor = VPIGetEditorFromID(id, editor_id);
+        if(editor == NULL)
+            return(VPI_FALSE);
+
+	return((editor->has_changes) ? VPI_TRUE : VPI_FALSE);
+}
+
+/*
+ *	Sets the has_changes flag on the editor.
+ */
+void VPIEditorSetHasChanges(vpi_id *id, int editor_id, int has_changes)
+{
+	gboolean b = (has_changes) ? TRUE : FALSE;
+        ma_editor_struct *editor = VPIGetEditorFromID(id, editor_id);
+        if(editor == NULL)
+            return;
+
+	if(editor->has_changes != b)
+	{
+	    editor->has_changes = b;
+
+	    /* Update menus. */
+	    EditorUpdateMenus(editor);
+	    EditorUpdateAllViewMenus(editor);
+	}
+}
+
 
 /*
  *	Returns a statically allocated string describing the given
@@ -2895,16 +3033,16 @@ FILE *VPICreateTemporyFile(
 )
 {
 	vma_core_struct *core_ptr;
-	vma_plugin_struct *pi = (vma_plugin_struct *)id;
+	vma_plugin_struct *pi = VMA_PLUGIN(id);
 
 
 	if(new_path_rtn != NULL)
-	    (*new_path_rtn) = NULL;
+	    *new_path_rtn = NULL;
 
 	if((pi == NULL) || (name_prefix == NULL))
 	    return(NULL);
 
-	core_ptr = (vma_core_struct *)pi->core_ptr;
+	core_ptr = VPIGetCoreFromID(id);
 
 	return(VMAMakeOpenTmpFile(
 	    core_ptr,
