@@ -234,7 +234,6 @@ typedef struct {
 #define VPI_FLAG_INIT_FILENAME		(1 << 1)
 #define VPI_FLAG_INIT_ARGC		(1 << 2)
 #define VPI_FLAG_INIT_ARGV		(1 << 3)
-
 	vpi_flag flags;
 
 #define VPI_INIT_START_TYPE_STANDARD	0
@@ -254,9 +253,13 @@ typedef struct {
  */
 typedef struct {
 
-        /* Defines which members of this structure are set. */
+	/* Defines which members of this structure are set. */
+#define VPI_FLAG_MANAGE_CLIENT_DATA	(1 << 0)
         vpi_flag flags;
 
+	/* Client data (set by VPISetClientData()). */
+	void *client_data;
+
 } vpi_manage_struct;
 
 /*
@@ -266,12 +269,16 @@ typedef struct {
 
         /* Defines which members of this structure are set. */
 #define VPI_FLAG_SHUTDOWN_REASON	(1 << 0)
+#define VPI_FLAG_SHUTDOWN_CLIENT_DATA	(1 << 1)
         vpi_flag flags;
 
 #define VPI_SHUTDOWN_REASON_NORMAL_REQUEST	0
 #define VPI_SHUTDOWN_REASON_ERROR_GENERAL	1
 	int reason;
 
+        /* Client data (set by VPISetClientData()). */
+        void *client_data;
+
 } vpi_shutdown_struct;
 
 
@@ -324,6 +331,16 @@ VPI_PROTO_RETURN_SHUTDOWN VPIShutdown(VPI_PROTO_INPUT_SHUTDOWN);
 #define VPI_PROTO_RETURN_SHUTDOWN	void
 
 
+/*
+ *	Sets the client data.
+ */
+extern void VPISetClientData(vpi_id *id, void *client_data);
+
+/*
+ *	Gets the client data.
+ */
+extern void *VPIGetClientData(vpi_id *id);
+
 
 /* ***********************************************************************
  *                          VERSION SETTING
@@ -349,10 +366,10 @@ extern int VPISetPluginVersion(
 
 /* Allocates and deallocates VPI data type structures.
  *
- * For all *New() functinos here, return of NULL means error.
+ * For all *New() functions here, return of NULL means error.
  *
- * For all *Delete() functions here, any allocated members on the passed
- * structure will be deallocated if set.
+ * For all *Delete() functions here, any allocated members on the given
+ * structure will be deallocated.
  */
 extern vpi_ftype_struct *VPIFTypeNew(void);
 extern void VPIFTypeDelete(vpi_ftype_struct *buf);
@@ -487,12 +504,18 @@ extern int VPIFileSelect(
  *	same control level as the calling function. Can return NULL if
  *	user canceled or response not available.
  */
-char *VPIFileSelectSimple(
+extern char *VPIFileSelectSimple(
         vpi_id *id, int editor_id,
         const char *path                /* Initial path, can be NULL. */
 );
 
 
+/*
+ *	Returns the pointer to the editor's toplevel GtkWidget, the
+ *	returned pointed will be a GtkWindow *.
+ */
+extern void *VPIGetEditorToplevel(vpi_id *id, int editor_id);
+
 
 /* ***********************************************************************
  *                  VALUE GET/FETCH ABSTRACTION LAYER
@@ -997,12 +1020,11 @@ extern void VPIEditorModelDelete(
 );
 
 /*
- *	Instructs editor to (re)select the specified model if it is valid.
- *	If model_num is -1 then model is unselected.
+ *	Instructs the editor to (re)select the specified model if it is
+ *	valid.  If model_num is -1 then the model is unselected.
  */
 extern void VPIEditorModelSelect(
-        vpi_id *id, int editor_id,
-        int model_num
+        vpi_id *id, int editor_id, int model_num
 );
 /*
  *	Returns the current selected model on the editor or -1.
@@ -1011,6 +1033,40 @@ extern v3d_model_struct *VPIEditorModelGetSelected(
         vpi_id *id, int editor_id, int *model_num
 );
 
+/*
+ *      Instructs the editor to (re)select the specified primitive if it
+ *	is valid.  If pn is -1 then all primitives will be unselected.
+ */
+extern void VPIEditorPrimitiveSelect(
+        vpi_id *id, int editor_id, int pn
+);
+/*
+ *	Returns the curret selected primitives on the editor or NULL.
+ *	The returned pointer must be free'ed by the calling function.
+ */
+extern int *VPIEditorPrimitiveGetSelected(
+        vpi_id *id, int editor_id, int *total
+);
+
+
+/*
+ *      Redraw the editor.
+ *
+ *      If refresh_lists is VPI_TRUE then the lists will be refreshed.
+ */
+extern void VPIEditorRedraw(vpi_id *id, int editor_id, int refresh_lists);
+
+/*
+ *      Returns VPI_TRUE if the editor's has_changes flag is set,
+ *      otherwise returns VPI_FALSE.
+ */
+extern int VPIEditorGetHasChanges(vpi_id *id, int editor_id);
+
+/*
+ *      Sets the has_changes flag on the editor.
+ */
+extern void VPIEditorSetHasChanges(vpi_id *id, int editor_id, int has_changes);
+
 
 
 /* ***********************************************************************
