@@ -233,8 +233,6 @@ static void VPIUnloadOSPlugin(void *handle)
 	    error_strptr = dlerror();	/* Just to be formal and sync in case. */
 	}
 #endif	/* __linux__ or __SOLARIS__ */
-
-	return;
 }
 
 /*
@@ -246,7 +244,6 @@ static void VPITouch(vma_plugin_struct *pi)
 	{
 	    pi->time_last_used = (time_t)time(NULL);
 	}
-	return;
 }
 
 
@@ -334,8 +331,10 @@ int VPIDisable(vma_plugin_struct *pi)
                 VPIGetOSPluginFunction(handle, VPI_PROTO_NAME_SHUTDOWN);
             if(shutdown_func != NULL)
             {
-                v.flags = (VPI_FLAG_SHUTDOWN_REASON);
+                v.flags =       VPI_FLAG_SHUTDOWN_REASON |
+                                VPI_FLAG_SHUTDOWN_CLIENT_DATA;
                 v.reason = VPI_SHUTDOWN_REASON_NORMAL_REQUEST;
+                v.client_data = pi->client_data;
 
                 pi->processing = 1;
                 shutdown_func((vpi_id *)pi, &v);
@@ -475,7 +474,7 @@ vma_plugin_struct *VPILoad(
 
 	    /* Plug-in loaded successfully. */
 	    if(error_code != NULL)
-		(*error_code) = 0;
+		*error_code = 0;
 	}
 	return(pi);
 }
@@ -510,8 +509,10 @@ void VPIUnload(vma_plugin_struct *pi)
 		VPIGetOSPluginFunction(handle, VPI_PROTO_NAME_SHUTDOWN);
 	    if(shutdown_func != NULL)
 	    {
-		v.flags = (VPI_FLAG_SHUTDOWN_REASON);
+		v.flags =	VPI_FLAG_SHUTDOWN_REASON |
+				VPI_FLAG_SHUTDOWN_CLIENT_DATA;
 		v.reason = VPI_SHUTDOWN_REASON_NORMAL_REQUEST;
+		v.client_data = pi->client_data;
 
                 pi->processing = 1;
 		shutdown_func((vpi_id *)pi, &v);
